# -*- coding: utf-8 -*-

""" Draw the Population graph.  Nodes will be drawn with a color corresponding
to the type_colors property of the Cell type used.

NOTE: This action requires Matplotlib

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import csv

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise ImportError("DrawPopulation requires matplotlib")

import networkx as nx
from seeds.Action import *

class DrawPopulation(Action):
    """ Draw the Population graph

    Configuration parameters for this action are set in the
    [DrawPopulation] section.

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        Base name for files.  The epoch at which the file was created and
        the extension (see format) will also comprise the resulting file
        name.  For example, a filename of 'population' when run at epoch
        1200 and using format pdf would produce the file
        population-001200.pdf.  (default: 'population')
    format
        The format to be used for resulting images.  Available formats
        depend on the particular installation of matplotlib, but often
        allow png, pdf, jpg, eps, and svg.  (default: 'png')
    transparent
        Whether or not the canvas (background of the image) will be
        transparent in the resulting images.  If not, the background will
        be white. (default: False)
    display_epoch
        Whether or not to include the epoch in text with the figure (default:
        False)

    Configuration Example:

    [DrawPopulation]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = population
    format = png
    transparent = True

    """
    def __init__(self, experiment, label=None):
        """Initialize the DrawPopulation Action"""

        super(DrawPopulation, self).__init__(experiment, name="DrawPopulation", label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', default='population')
        self.format = self.experiment.config.get(self.config_section, 'format', default='png')
        self.transparent = self.experiment.config.getboolean(self.config_section, 'transparent', default=False)
        self.display_epoch = self.experiment.config.getboolean(self.config_section, 'display_epoch', default=False)

        self.graph = self.experiment.population.topology.graph
        self.colors = self.experiment.population._cell_class.type_colors

        # Get the coordinates of each node
        self.pos = {}
        for n in self.graph.nodes():
            x = self.graph.node[n]['coords'][0]
            y = self.graph.node[n]['coords'][1]
            self.pos[n] = [x,y]

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        # Get a list of the colors to use for each node
        cols = []
        for n in self.graph.nodes():
            cols.append(self.colors[self.graph.node[n]['cell'].type])

        plt.figure()
        nx.draw(self.graph, with_labels=False,
                pos=self.pos, edge_color='#777777', node_color=cols, node_size=40)

        if self.display_epoch:
            plt.text(0, -0.05, "Epoch: %d" % (self.experiment.epoch),
                     horizontalalignment='left',
                     size='small')

        filename = "%s-%06d.%s" % (self.filename, self.experiment.epoch, self.format)
        data_file = self.datafile_path(filename)
        plt.savefig(data_file, transparent=self.transparent)
        plt.close()



# -*- coding: utf-8 -*-
"""
Collection of functions that implement commonly-used numeric algorithms
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


def is_numeric(s):
    """Test whether or not a given value is numeric (integer or float)"""

    try:
        float(s)
        return True
    except ValueError:
        return False
    except TypeError:
        return False



# -*- coding: utf-8 -*-

""" This template outlines the necessary components for implementing a new Action
in SEEDS.  Generally, this involves implementing a constructor (__init__) and
an update method.

Areas marked with 'TODO' should be replaced with code specific to the Action
being implemented.

The name of the Action type should be the name of the class.  The name of the
file should also match this.

Methods and parameters common to all Action objects can be seen in the
Action.py file in the main SEEDS codebase.  A good example Action would be the
PrintCellTypeCount action located in plugins/actions.

Once completed, new Action type files can be placed in a plugins directory and
used by appending the name of the Action to the "actions" parameter in the
[Experiment] section of the configuration file.

"""

__author__ = "TODO"
__credits__ = "TODO"


# TODO: if you'll be writing data files, you'll want to use the csv module.
# All data files created by SEEDS should be in csv format.
import csv

from seeds.Action import *

class TODO-ActionName(Action):
    """ TODO: add documentation for the action describing what it's done and
    how it's configured.
    """

    def __init__(self, experiment, label=None):
        """Initialize the Action"""

        # Call the Action constructor.  This will set various parameters common
        # to all actions, such as defining the self.config_section parameter,
        # used below.
        super(TODO-ActionName, self).__init__(experiment,
                                                 name="TODO-ActionName",
                                                 label=label)

        # TODO: all actions have the following configuration parameters
        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)

        # TODO: load and validate any other configuration parameters here

        # TODO: if you plan to have one output file for this action, create and
        # open it here.


    # TODO: the update method is called at each epoch and performs whatever
    # tasks are associated with the action (e.g., writing a row of data,
    # creating a plot, killing organisms, etc.).
    def update(self):
        """Execute the action"""

        # NOTE: this code can be kept here.  It allows actions to be run at
        # certain intervals by using the epoch_start, epoch_end, and frequency
        # parameters.
        if self.skip_update():
	        return

        # TODO: perform the tasks associated with this action.




# -*- coding: utf-8 -*-
"""
Interface for Topologies.

A topology is a graph that defines the interactions between nodes.  Each node
contains one Cell or one Resource.  If a pair of nodes is connected, the Cells
or Resources housed in those nodes are thought of as "neighbors", and therefore
can potentially interact with each other.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

from math import sqrt

import networkx as nx
from networkx.exception import *

from seeds.SEEDSError import *
from seeds.utils.geometry import euclidean_distance


class Topology(object):
    """
    All topologies contain properties:

        config_section
            The name of the section in the configuration file that stores the
            configuration for this Topology.  This will likely need to be set
            by each Topology plugin (e.g., CartesianTopology:label1).
        graph
            A NetworkX graph object defining the connections between cells.
        experiment
            Reference to the Experiment in which it exists
        periodic
            Whether or not to use periodic boundary conditions, which connects
            the edges of the space (default: False)
        label
            A unique label identifying a configuration for the Topology

    """

    def __init__(self, experiment, label=None):
        """Initialize a Topology object.

        The topology will have no cells and an empty graph

        Parameters:

        *experiment*
            A reference to the Experiment
        *label*
            A unique label identifying a configuration for the Topology

        """

        self.experiment = experiment
        self.graph = nx.Graph()
        self.periodic = False
        self.label = label
        self.config_section = None

    def __str__(self):
        """Return a string to be used when a Topology object is printed"""
        return 'SEEDS Topology'

    def get_neighbors(self, node):
        """Get a list of neighboring nodes (ids) for a given node

        Parameters:
        
        *node*
            The ID of the node whose neighboring cells to get
        
        """

        return self.graph.neighbors(node)

    def num_nodes(self):
        """Get the number of nodes in the topology"""
        return len(self.graph)

    def teardown(self):
        """Perform any necessary cleanup at the end of the experiment"""
        pass

    def node_distance(self, src, dest):
        """Calculate the Euclidean distance between the given two nodes using
        their 'coords' properties

        Parameters:

        *src*
            The first node ID
        *dest*
            The second node ID

        If either node does not exist, NonExistentNodeError will be raised

        """

        if src not in self.graph.nodes():
            raise NonExistentNodeError(src)
        elif dest not in self.graph.nodes():
            raise NonExistentNodeError(dest)

        return euclidean_distance(self.graph.node[src]['coords'],
                                  self.graph.node[dest]['coords'],
                                  periodic=self.periodic)

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph.  Topologies that do not wish to support
        this should redefine this method to do nothing.  This method will
        not place a Cell or ResourceCell in the newly-created node.  That
        will need to be done separately.

        The general convention in SEEDS is for all nodes to have coordinates in
        unit Cartesian space.  These coordinates are stored as a tuple as a
        node property named 'coords'.  New nodes should be given coordinates
        appropriately.

        Example:

            self.graph.node[id]['coords'] = (0.21, 0.00313)

        Parameters:

        id
            The ID to use for the new node.  If none is specified (or -1), the
            ID used will be the current largest ID in the graph plus 1.
        neighbors
            An optional list of node IDs that will be connected to the new node
            via an edge. NonExistentNodeError will be raised if any of these
            nodes do not exist.

        """

        if id == -1:
            self.graph.add_node(max(self.graph.nodes()) + 1)
        else:
            self.graph.add_node(id)

        for n in neighbors:
            if n not in self.graph.nodes():
                raise NonExistentNodeError(n)
            self.graph.add_edge(id, n)

    def remove_node(self, id):
        """Remove a node from the graph.  Topologies that do not wish to
        support this should redefine this method to do nothing.  This method
        will not perform teardown actions for any Cell or ResourceCell objects
        residing in the node.  That will need to be done separately (before
        remove_node is called).  Any edges associated with the node will also
        be removed.

        Parameters:

        id
            The ID to use of the node to be deleted.

        """

        try:
            self.graph.remove_node(id)
        except NetworkXError as err:
            raise NonExistentNodeError(id)

    def add_edge(self, src, dest):
        """Add an edge between the given two nodes.  Although NetworkX creates
        new node(s) when non-existent nodes are given as arguments to
        add_edge(), this method will raise an exception (NonExistentNodeError)
        if either of the given nodes does not exist.

        Parameters:

        src
            ID of the source node
        dest
            ID of the dest node

        """

        if src not in self.graph.nodes():
            raise NonExistentNodeError(src)
        elif dest not in self.graph.nodes():
            raise NonExistentNodeError(dest)
        else:
            self.graph.add_edge(src, dest)

    def remove_edge(self, src, dest):
        """Remove the edge between the given two nodes.  This method will raise
        an exception (NonExistentEdgeError) if either of the given nodes does
        not exist.

        Parameters:

        src
            ID of the source node
        dest
            ID of the dest node

        """

        try:
            self.graph.remove_edge(src, dest)
        except NetworkXError as err:
            raise NonExistentEdgeError(src, dest)

    def get_nearest_node(self, coords, n=1):
        """Return a list of  the node(s) located nearest the given coordinates

        Parameters:

        coords
            Tuple defining the point in question.  The number of dimensions
            should match the number of dimensions of the topology.
        n
            The number of nearest neighbors to find

        """

        if not coords or len(coords) < 1:
            return
        elif n < 1:
            # TODO: print warning or throw an exception 
            # TODO: search the KD tree for the coordinates
            return




# -*- coding: utf-8 -*-
"""
Lattice topology in which each node is connected with each of its 8 neighbors
(Moore Neighborhood).  The radius of interactions can be defined, which means
all nodes within this many hops will be considered a neighbor.

This topology was originally presented and used used in the publication:

    B.D. Connelly, L. Zaman, C. Ofria, and P.K. McKinley, "Social structure and
    the maintenance of biodiversity," in Proceedings of the 12th International
    Conference on the Synthesis and Simulation of Living Systems (ALIFE), pp.
    461-468, 2010.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman, Philip McKinley, Charles Ofria"

from math import floor

import networkx as nx

from seeds.SEEDSError import *
from seeds.Topology import *


class MooreTopology(Topology):
    """
    Lattice topology with Moore Neighborhoods with configurable radius

    Nodes are organized on a lattice.  A node's neighbors reside to the
    left, right, above, below, and on the diagonals.  With radius 1, this
    neighborhood will contain 8 nodes.  With radius 2, the 24 nodes within
    a 2-hop distance are included.

    Configuration: All configuration options should be specified in the
        MooreTopology block (unless otherwise specified by the config_section
        parameter).

        size: Width/height, in number of nodes, of the Experiment.  With size
            10, there would be 100 nodes. (no default)
        periodic: Whether or not the Experiment should form a torus.
            This means that nodes on the left border are neighbors with
            nodes on the right border. (default: False)
        radius: Number of hops within a focal node's neighborhood
            (default: 1)

    Example:
        [MooreTopology]
        size = 100
        periodic = True
        radius = 4


    """
    def __init__(self, experiment, label=None):
        """Initialize a MooreTopology object"""
        super(MooreTopology, self).__init__(experiment, label=label)

        if self.label:
            self.config_section = "%s:%s" % ("MooreTopology", self.label)
        else:
            self.config_section = "%s" % ("MooreTopology")

        self.size = self.experiment.config.getint(self.config_section, 'size')
        self.periodic = self.experiment.config.getboolean(self.config_section, 'periodic', default=False)
        self.radius = self.experiment.config.getint(self.config_section, 'radius', default=1)

        if not self.size:
            raise ConfigurationError("MooreTopology: size parameter must be defined")
        elif self.size < 1:
            raise ConfigurationError("MooreTopology: size must be positive")
        elif self.radius < 0:
            raise ConfigurationError("MooreTopology: radius must b greater than or equal to 0")
        elif self.radius >= self.size:
            raise ConfigurationError("MooreTopology: radius can not exceed grid size")

        self.graph = self.moore_2d_graph(self.size, self.size,
                                         radius=self.radius,
                                         periodic=self.periodic)

        for n in self.graph.nodes():
            self.graph.node[n]['coords'] = (self.row(n)/float(self.size), self.column(n)/float(self.size))

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'Moore Topology (%d nodes, %d radius)' % (self.size * self.size, self.radius)

    def row(self, nodeid):
        """Get the number of the row in which the given node is located

        Parameters:

        *nodeid*
            The ID of the node in question

        """
        return (nodeid//self.size)

    def column(self, nodeid):
        """Get the number of the column in which the given node is located

        Parameters:

        *nodeid*
            The ID of the node in question

        """
        return nodeid % self.size

    def node_id(self, row, col):
        """Get the ID of the node at the given row and column

        Parameters:

        *row*
            The row in which the node is located
        *col*
            The column in which the node is located

        """
        return row * self.size + col

    def moore_2d_graph(self, rows=0, columns=0, radius=0,
                       periodic=False):
        """ Return the 2d grid graph of rows x columns nodes,
            each connected to its nearest Moore neighbors within
            a given radius.
            Optional argument periodic=True will connect
            boundary nodes via periodic boundary conditions.

        Parameters:

        *rows*
            The number of rows to be in the graph
        *columns*
            The number of columns to be in the graph
        *radius*
            The radius of interactions in the graph (there will be an edge
            between a node and all other nodes within N hops)
        *periodic*
            Prevent edge effects using periodic boundaries

        """
        G = nx.empty_graph()
        G.name = "moore_2d_radius_graph"
        G.add_nodes_from(list(range(rows * columns)))

        for n in G.nodes():
            myrow = self.row(n)
            mycol = self.column(n)

            for r in range(myrow - radius, myrow + radius + 1):
                if periodic == False and (r < 0 or r >= rows):
                    continue

                for c in range(mycol - radius, mycol + radius + 1):
                    if periodic == False and (c < 0 or c >= columns):
                        continue

                    nid = self.node_id(r % rows, c % columns)

                    if nid != n:
                        neighbor_id = ((r % rows) * self.size) + (c % columns)
                        G.add_edge(n, neighbor_id)

        return G

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_node is not supported by MooreTopology")
        return

    def remove_node(self, id):
        """Remove a node from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_node is not supported by MooreTopology")
        return

    def add_edge(self, src, dest):
        """Add an edge to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_edge is not supported by MooreTopology")
        return

    def remove_edge(self, src, dest):
        """Remove an edge from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_edge is not supported by MooreTopology")
        return


    def get_nearest_node(self, coords, n=1):
        """Return a list of the node(s) located nearest the given coordinates

        Note that this overwrites the get_nearest_node method defined by
        Topology (due to its simplicity vs using a k-d Tree).  This specific
        implementation may be removed in future releases if the more complete
        version in Topology is found to be better.

        Parameters:

        coords
            Tuple defining the point in question.  The number of dimensions
            should match the number of dimensions of the topology.
        n
            The number of nearest neighbors to find

        """

        if not coords or len(coords) < 1:
            print("ERROR: Invalid coordinates for get_nearest_node")
            return
        elif n < 1:
            print("ERROR: Invalid number of neighbors for get_nearest_node")
            return

        cell_width = 1.0 / self.size

        nearest_x = floor(float(coords[0]) / cell_width) * cell_width
        nearest_y = floor(float(coords[1]) / cell_width) * cell_width

        nearest_col = int(floor(coords[0] / cell_width))
        nearest_row = int(floor(coords[1] / cell_width))
        nearest_node = self.node_id(nearest_row, nearest_col)

        nearest = [nearest_node]

        for i in range(n - 1):
            print("ERROR: get_nearest_node only supports 1 neighbor at the moment")
            # TODO: look at neighbors of calculated nearest point and expand out.  Append to nearest.

        return nearest



# -*- coding: utf-8 -*-
""" Cell type representing the classic game Rock-Paper-Scissors (RPS).  In this
system, a cell can be either a rock, paper, or scissors.  Rock is defeated by
paper, paper is defeated by scissors, and scissors is defeated by rock.  This
is a non-transitive system, where each strategy beats one but is beaten by
another.  If any one strategy is lost, another strategy will also be lost, as
the non-transitivity is broken (e.g., if no one can play rock, scissors will
always win the game).

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import random

from seeds.Cell import *
from seeds.SEEDSError import *
from seeds.utils.sampling import roulette_select


class RPSCell(Cell):
    """
    This is a simple Cell type modeling the classic Rock-Paper-Scissors (RPS)
    game.

    This Cell type has 3 states:
        ROCK: Outcompetes SCISSORS cells
        PAPER: Outcompetes ROCK cells
        SCISSORS: Outcompetes PAPER cells

    Configuration: The following configuration parameters can be configured in
    the [RPSCell] section of the config.

    distance_dependent
        Whether or not a Cell is more likely to interact with nearby
        neighboring Cells.  In this case, the probability of interacting with a
        given neighbor is proportional to the distance to that neighbor.
        (Default: False)

    """

    types = ['Rock', 'Paper', 'Scissors']
    type_colors = ['r','g','b']
    max_types = 3

    ROCK = 0
    PAPER = 1
    SCISSORS = 2

    def __init__(self, experiment, population, id, type=-1, name="RPSCell", label=None):
        """Initialize a RPSCell object

        The type for the cell is selected at random.

        Parameters:

        *experiment*
            A reference to the Experiment
        *population*
            A reference to the Population in which this Cell exists
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)
        *name*
            The name of this Cell type
        *label*
            A unique label for configuring this Cell type

        """

        super(RPSCell, self).__init__(experiment, population, id, name=name, label=label)

        if type == -1:
            self.type = random.randint(0, len(self.types)-1)
        else:
            self.type = type
        
        self.population.increment_type_count(self.type)

        self.distance_dependent = self.experiment.config.getboolean(section=self.config_section,
                                                                    name='distance_dependent',
                                                                    default=False)

    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'RPSCell %d Type %d (%s)' % (self.id, self.type, self.types[self.type])

    def type(self):
        """Return the name of the type of this cell"""
        return self.types[self.type]

    def update(self):
        """Update the cell based on a competition with a randomly-selected
        neighbor

        Rock cells will be replaced by paper cells.  Paper cells will be
        replaced by Scissors cells.  Scissors cells will be replaced by Rock
        cells.

        If a Cell has no neighbors, the Cell can not be updated.  In this case,
        a warning is printed.

        """

        neighbors = self.get_neighbors()

        if len(neighbors) < 1:
            print("Warning: Can not update RPSCell with 0 neighbors")
            return

        if self.distance_dependent:
            # Select a competitor with probability proportional to the
            # closeness of that neighbor (roulette wheel)

            # Adding a very small number to the distances to prevent
            # divide-by-zero errors, which can occur in well-mixed topologies,
            # where a Cell can exist in its own neighbor list

            distances = self.get_neighbor_distances()
            inv_dist = [1.0/(d + pow(1.02,-10000)) for d in distances]
            competitor = roulette_select(items=neighbors, fitnesses=inv_dist, n=1)[0]
        else:
            # Pick a random neighbor to compete with.  If that neighbor wins, it
            # gets the current cell.
            competitor = random.choice(neighbors)

        if self.type == self.ROCK and competitor.type == self.PAPER:
            self.type = self.PAPER
            self.population.update_type_count(self.ROCK, self.type)            
        elif self.type == self.PAPER and competitor.type == self.SCISSORS:
            self.type = self.SCISSORS
            self.population.update_type_count(self.PAPER, self.type)            
        elif self.type == self.SCISSORS and competitor.type == self.ROCK:
            self.type = self.ROCK
            self.population.update_type_count(self.SCISSORS, self.type)            





# -*- coding: utf-8 -*-
"""
Print the coordinates of each cell and its type for each world
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import csv

from seeds.Action import *

class PrintCellLocations(Action):
    """ Write the x,y coordinates of each cell and its type

    Configuration parameters for this action are set in the
    [PrintCellLocations] section.

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        Base name for files.  The epoch at which the file was created and the
        extension (see format) will also comprise the resulting file name.  For
        example, a filename of 'cell_locations' when run at epoch 1200 would
        produce the file cell_locations-001200.pdf.  (default:
        'cell_locations')
    header
        Whether or not to write a header to the output file.  The header will
        be an uncommented, comma-separated list of property names corresponding
        to the data in each row. (default: True)

    Configuration Example:

    [PrintCellLocations]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = cell_locations
    header = True

    """
    def __init__(self, experiment, label=None):
        """Initialize the PrintCellLocations Action"""
        super(PrintCellLocations, self).__init__(experiment, name="PrintCellLocations", label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'cell_locations')
        self.header = self.experiment.config.getboolean(self.config_section, 'header', default=True)

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        filename = "%s-%06d.csv" % (self.filename, self.experiment.epoch)
        data_file = self.datafile_path(filename)
        self.writer = csv.writer(open(data_file, 'w'))

        if self.header:
            header = ['epoch','cell_id','x','y','type']
            self.writer.writerow(header)

        g = self.experiment.population.topology.graph
        for n in g.nodes():
            cell = g.node[n]['cell']
            (xpos, ypos) = cell.coords()

            row = [self.experiment.epoch, cell.id, xpos, ypos, cell.type]
            self.writer.writerow(row)




# -*- coding: utf-8 -*-
"""
Interface for Actions.

An Action is a piece of code that is executed at specified epochs and perform
things such as writing statistics, plotting graphs, or interacting with
Topologies, Cells, or Resources.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman"

import os


class Action(object):
    """
    Properties:

    experiment
        A reference to the Experiment
    data_dir
        Directory in which to store created files
    epoch_start
        The epoch at which the Action starts executing
    epoch_end
        The epoch at which the Action stops executing
    frequency
        The frequency at which the Action is executed.  For example, if
        frequency=2, then the Action is executed at every other epoch.
    name
        The name of the action
    label
        A unique label for the Action.  Used when multiple instances of one
        action are used.
    priority
        The priority of the action.  This is useful for actions for which it is
        important to execute in a specific order.  Larger values mean higher
        priority.  Default: 0.
    enabled
        Whether or not the Action should be run.  (Boolean, Default: True)
    header
        For Actions that write data files, whether or not to write a header
        row.  (Boolean, Default: True)
    
    Configuration: The data_dir parameter should be set in the [Experiment]
    block.  Each Action should have its own configuration block.

    """

    def __init__(self, experiment, name=None, label=None):
        """Create an Action instance"""
        self.experiment = experiment
        self.name = name
        self.label = label
        self.data_dir = self.experiment.config.get(self.experiment.config_section, 'data_dir', 'data')
        self.epoch_start = 0
        self.epoch_end = self.experiment.config.getint(self.experiment.config_section, 'epochs', default=-1)
        self.frequency = 1
        self.priority = 0
        self.enabled = True
        self.header = True

        self.config_section = self.get_config_section()

    def __str__(self):
        """Produce a string to be used when an Action object is printed"""
        return 'Action Object (%s:%s) (epoch start: %d)(epoch end: %d)(frequency: %d)' % (self.name, self.label, self.epoch_start, self.epoch_end, self.frequency)

    def update(self):
        """ Execute the action """
        pass

    def teardown(self):
        """Perform any necessary cleanup at the end of the experiment"""
        pass

    def skip_update(self):
        """ Return a boolean indicating whether or not the action should be
        executed during the current epoch

        """

        return (self.experiment.epoch < self.epoch_start or
                (self.epoch_end != -1 and self.experiment.epoch > self.epoch_end) or
                (self.experiment.epoch - self.epoch_start) % self.frequency != 0 or
                not self.enabled)

    def datafile_path(self, filename):
        """ Return the relative path to a given data file
        Parameters:

        *filename*
            The name of the file whose path to get

        """
        return os.path.join(self.data_dir, filename)

    def get_config_section(self):
        """ Return a string containing the configuration file section for this
        action.  This section is composed of <name>:<label>

        """

        if self.label:
            r = "%s:%s" % (self.name, self.label)
        else:
            r = self.name

        return r




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Produce plots (using matplotlib) from data written by the SEEDS
PrintPopulationTypeClusters action.  This can either be used as a command-line
tool or by importing and calling the plot_num_clusters function.

Dependencies:
    - Python 2.7 or greater
    - Matplotlib
    - Numpy
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__version__ = "1.0"
__credits__ = "Brian Connelly"

import argparse
import csv
import numpy
import re
import string
import sys

import matplotlib.pyplot as plt

__author__ = "Brian Connelly <bdc@msu.edu>"
__version__ = "1.0"


def plot_num_clusters(file, outfile='num_clusters.pdf', title=None, grid=False, epochs=[],
                      labels=[]):
    reader = csv.reader(file)
    rownum = 0
    data_read = False

    for row in reader:
        # Skip commented lines
        if re.match('^\s*#', row[0]) != None:
            continue

        rownum += 1

        if rownum == 1:
            colnames = row[1:]

            for i in range(len(colnames)):
                colnames[i] = re.sub('_', ' ', colnames[i])

            continue
        else:
            row = list(map(float, row))

            if (len(epochs) == 0 or row[0] in epochs):
                if data_read:
                    data = numpy.vstack((data,row))
                else:
                    data = numpy.array(row)
                    data_read = True

    if data_read:
        fig = plt.figure()

        if len(labels) > 0 and len(labels) == len(colnames):
            colnames = labels

        plot_cols = list(range(1, data.shape[1], 3))

        # Plot the number of clusters
        for t in plot_cols:
            plt.errorbar(data[:,0], data[:,t], yerr=None, xerr=None, label=string.capitalize(colnames[t-1]))

        if grid:
            plt.grid()

        plt.xlabel("Time (epoch)")
        plt.ylabel("Clusters")

        if title:
            plt.title(title)

        plt.legend(loc=0)
        plt.savefig(outfile)
    else:
        print("Could not generate plot: No data match given parameters")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Plot population densities created from the PrintCellTypeCount SEEDS Action')
    parser.add_argument('infile', nargs='?', type=argparse.FileType('r'), default=sys.stdin)
    parser.add_argument('-e', '--epochs', action='store', default=[], nargs='+',
                        help='epochs for which to show data (default: all)')
    parser.add_argument('-g', '--grid', action='store_true', help='show a grid (default: False)')
    parser.add_argument('-o', '--outfile', action='store', default="num_clusters.pdf",
                        help='name of generated plot (default: num_clusters.pdf)')
    parser.add_argument('-l', '--labels', action='store', nargs='+', default=[],
                        help='list of labels for each cell type (default: use data file)')
    parser.add_argument('-t', '--title', action='store', default=None,
                        help='title of the plot (default: no title)')
    parser.add_argument('-v', '--version', action='version', version=__version__,
                        help='display version information and exit')
    args = parser.parse_args()

    epochs = []
    if len(args.epochs) > 0:
        for r in args.epochs:
            m = re.match(r"^\s*(?P<start>\d+)\-(?P<end>\d+)\s*$", r)
            if m != None:
                epochs += list(range(int(m.group("start")), int(m.group("end"))+1))

            m = re.match(r"^\s*(?P<value>\d+)\s*$", r)
            if m != None:
                epochs.append(int(m.group("value")))

    # Sort the list and remove duplicates
    epochs.sort()
    s = set(epochs)
    epochs = []
    [epochs.append(i) for i in s]

    plot_num_clusters(file=args.infile, outfile=args.outfile, title=args.title,
                      grid=args.grid, epochs=epochs, labels=args.labels)




# -*- coding: utf-8 -*-
"""Print statistics about the clusters of each Cell type and overall clustering
in the population.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import networkx as nx

import csv
import random

from seeds.Action import *
from seeds.utils.statistics import mean, std

class PrintPopulationTypeClusters(Action):
    """ Write a data file containing the numbers of clusters of each Cell type,
    as well as their mean size and the standard deviation in size

    Configuration is done in the [PrintPopulationTypeClusters] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        The name of the file to write to (default:
        population_type_clusters.csv)
    header
        Whether or not to write a header to the output file.  The header will
        be an uncommented, comma-separated list of property names corresponding
        to the data in each row. (default: True)


    Configuration Example:

    [PrintPopulationTypeClusters]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = population_type_clusters.csv
    header = True

    """
    def __init__(self, experiment, label=None):
        """Initialize the PrintPopulationTypeClusters Action"""

        super(PrintPopulationTypeClusters, self).__init__(experiment,
                                                          name="PrintPopulationTypeClusters",
                                                          label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'population_type_clusters.csv')
        self.header = self.experiment.config.get(self.config_section, 'header', default=True)
        self.name = "PrintPopulationTypeClusters"
        self.types = self.experiment.population._cell_class.types

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        if self.header:
            header = ['epoch', 'total_clusters', 'total_size_mean', 'total_size_std']
            for t in self.types:
                header.append('%s_clusters' % (t))
                header.append('%s_size_mean' % (t))
                header.append('%s_size_std' % (t))
            self.writer.writerow(header)
      
    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        g = self.experiment.population.topology.graph
        cluster_counts = [0] * len(self.types)
        cluster_sizes = {}

        for i in range(len(self.types) + 1):
            cluster_sizes[i] = []

        unvisited = g.nodes()

        while len(unvisited) > 0:
            node = random.choice(unvisited)
            type = g.node[node]["cell"].type

            c = cluster(g, node)
            c_size = len(c)
            cluster_counts[type] += 1
            cluster_sizes[type].append(c_size)
            cluster_sizes[len(self.types)].append(c_size)

            [unvisited.remove(x) for x in c]

        row = [self.experiment.epoch]
        row.append(sum(cluster_counts))
        row.append(mean(cluster_sizes[len(self.types)]))
        row.append(std(cluster_sizes[len(self.types)]))

        for t in range(len(self.types)):
            row.append(cluster_counts[t])
            row.append(mean(cluster_sizes[t]))
            row.append(std(cluster_sizes[t]))

        self.writer.writerow(row)

def cluster(graph, node):
    visited_nodes = []

    def visit(node):
        visited_nodes.append(node)
        for n in graph.neighbors(node):
            if n not in visited_nodes and graph.node[n]['cell'].type == graph.node[node]['cell'].type:
                visit(n)

    visit(node)
    return visited_nodes




# -*- coding: utf-8 -*-
"""
Well-mixed topology in which each node is a neighbor of every other node.

This topology is modeled as a graph with no edges between the nodes.  Although
a complete graph would perhaps model a well-mixed topology more accurately,
this would prohibit large populations due to the associated memory
requirements.

Nodes are given a randomly-assigned location to more easily allow for
visualization.  It should be noted that in this topology, interactions
are not localized, so the neighbors with which a node interacts will
be located throughout the environment.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import random
import networkx as nx

from seeds.SEEDSError import *
from seeds.Topology import *


class WellMixedTopology(Topology):
    """
    Well-mixed topology with a configurable number of interactions

    Configuration: All configuration options should be specified in the
        WellMixedTopology block (unless otherwise specified by the
        config_section parameter).

        size
            Total number of nodes in the population
        num_interactions
            When a node is updated, a list of neighbors is passed to it.
            This parameter specifies the size of a random subset of nodes
            to be used as this neighbor set.  By default, all nodes in
            the populaion are passed as neighbors.

    Example:
        [WellMixedTopology]
        size = 100000
        num_interactions = 10


    """
    def __init__(self, experiment, config_section='WellMixedTopology'):
        """Initialize a WellMixedTopology object"""
        super(WellMixedTopology, self).__init__(experiment, config_section=config_section)

        self.size = self.experiment.config.getint(section=self.config_section,
                                                  name='size')
        self.num_interactions = self.experiment.config.getint(section=self.config_section,
                                                              name='num_interactions',
                                                              default=self.size)
        if not self.size:
            raise ConfigurationError("WellMixedTopology: size must be defined")
        elif self.size < 1:
            raise ConfigurationError("WellMixedTopology: size must be greater than 0")
        elif self.num_interactions < 0:
            raise ConfigurationError("WellMixedTopology: num_interactions must be non-negative")
        elif self.num_interactions > self.size:
            raise ConfigurationError("WellMixedTopology: num_interactions can not exceed size")

        self.graph = nx.empty_graph()
        self.graph.name = "well_mixed_graph"
        self.graph.add_nodes_from(list(range(self.size)))

        for n in self.graph.nodes():
            self.graph.node[n]['coords'] = (random.random(),random.random())

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return "Well-Mixed Topology (%d nodes, %d interactions)" % (self.size, self.num_interactions)


    def get_neighbors(self, node):
        """Get a randomly-selected list of neighboring nodes (IDs) for a given node

        Parameters:

        *node*
            The ID of the node whose neighboring nodes to get

        """

        return random.sample(self.graph.nodes(), self.num_interactions)

    def add_edge(self, src, dest):
        """Add an edge to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_edge is not supported by WellMixedTopology")
        return

    def remove_edge(self, src, dest):
        """Remove an edge from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_edge is not supported by WellMixedTopology")
        return

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph.  Topologies that do not wish to support
        this should redefine this method to do nothing.  This method will
        not place a Cell or ResourceCell in the newly-created node.  That
        will need to be done separately.

        Note that since edges aren't used in this topology, the neighbors
        argument is ignored.

        Parameters:

        id
            The ID to use for the new node.  If none is specified (or -1), the
            ID used will be the current largest ID in the graph plus 1.
        neighbors
            An optional list of node IDs that will be connected to the new node
            via an edge. NonExistentNodeError will be raised if any of these
            nodes do not exist. ***This argument is ignored***

        """

        if id == -1:
            self.graph.add_node(max(self.graph.nodes()) + 1)
        else:
            self.graph.add_node(id)

        self.graph.node[id]['coords'] = (random.random(),random.random())




# -*- coding: utf-8 -*-

""" Illustrate the transitions between cell types.  Nodes will be drawn with a
color corresponding to the type_colors property of the Cell type used.  A
directed edge between two nodes represents that the source node is losing (net)
cells to the destination node at that point in time.  The weight of the edge
represents the magnitude of this transfer.

NOTE: This action requires Matplotlib

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import csv
import math

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise ImportError("DrawTransitions requires matplotlib")

import networkx as nx
from seeds.Action import *

class DrawTransitions(Action):
    """ Draw the transitions between cell types

    Configuration parameters for this action are set in the
    [DrawTransitions] section.

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        Base name for files.  The epoch at which the file was created and the
        extension (see format) will also comprise the resulting file name.  For
        example, a filename of 'transitions' when run at epoch 1200 and using
        format pdf would produce the file transitions-001200.pdf.  (default:
        'transitions')
    format
        The format to be used for resulting images.  Available formats
        depend on the particular installation of matplotlib, but often
        allow png, pdf, jpg, eps, and svg.  (default: 'png')
    transparent
        Whether or not the canvas (background of the image) will be
        transparent in the resulting images.  If not, the background will
        be white. (default: False)
    display_epoch
        Whether or not to include the epoch in text with the figure (default:
        False)

    Configuration Example:

    [DrawTransitions]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = population
    format = png
    transparent = True

    """
    def __init__(self, experiment, label=None):
        """Initialize the DrawTransitions Action"""

        super(DrawTransitions, self).__init__(experiment, name="DrawTransitions", label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', default='transitions')
        self.format = self.experiment.config.get(self.config_section, 'format', default='png')
        self.transparent = self.experiment.config.getboolean(self.config_section, 'transparent', default=False)
        self.display_epoch = self.experiment.config.getboolean(self.config_section, 'display_epoch', default=False)

        self.colors = self.experiment.population._cell_class.type_colors
        self.types = self.experiment.population._cell_class.types
        self.max_types = self.experiment.population._cell_class.max_types

        self.node_labels = {}

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        edge_colors = []

        graph = nx.DiGraph()
        for i in range(self.max_types):
            graph.add_node(i)

        # Get a list of the sizes to use for each node
        node_sizes = []
        min_size = 10
        max_size = 35000

        # NOTE: this size is radius-based.  Do area-based for more accurate analogy?
        for i in range(self.max_types):
            count = float(self.experiment.data['population']['type_count'][i])
            frac = count / len(self.experiment.population.topology.graph)
            mysize = min_size + (frac * (max_size - min_size))
            node_sizes.append(mysize)

        if self.experiment.epoch > 0:
            # Get a list of the colors to use for each edge.
            edge_colors = []
            tmatrix = self.experiment.data['population']['transitions']

            for f in range(self.max_types):
                for t in range(self.max_types):
                    if tmatrix[f][t] > tmatrix[t][f]:
                        edge_colors.append(self.colors[f])
                        wt = tmatrix[f][t] - tmatrix[t][f]
                        graph.add_edge(f, t, weight=wt)

        # Make the figure
        plt.figure()
        nx.draw(graph, with_labels=False, pos=nx.circular_layout(graph),
                edge_color=edge_colors, node_color=self.colors, node_size=node_sizes, labels=self.node_labels)

        if self.display_epoch:
            plt.text(0, -0.05, "Epoch: %d" % (self.experiment.epoch),
                     horizontalalignment='left',
                     size='small')

        filename = "%s-%06d.%s" % (self.filename, self.experiment.epoch, self.format)
        data_file = self.datafile_path(filename)
        plt.savefig(data_file, transparent=self.transparent)
        plt.close()



# -*- coding: utf-8 -*-
""" Print detailed information about the Experiment and the software
environment under which the Experiment was performed.  This action is intended
to aide in recreating and reproducing Experiments.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import datetime
import hashlib
import networkx as nx
import os
import seeds as s
import sys

from seeds.Action import *
from seeds.SEEDSError import *


class PrintExperimentInformation(Action):
    """ Write detailed information about the Experiment and the software
    environment under which it was performed.  Generally, this Action only
    needs to be run once per experiment.

    Configuration parameters for this action are set in the
    [PrintExperimentInformation] section.

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        Name of the file to be written to (default: information.txt)

    Configuration Example:

    [PrintExperimentInformation]
    epoch_start = 0
    epoch_end = 0
    frequency = 1
    priority = 0
    filename = information.txt

    """

    def __init__(self, experiment, label=None):
        """Initialize the PrintExperimentInformation Action"""

        super(PrintExperimentInformation, self).__init__(experiment,
                                                         name="PrintExperimentInformation",
                                                         label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', default=0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=0)
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', default=1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', default=0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'information.txt')
        self.name = "PrintExperimentInformation"

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        data_file = self.datafile_path(self.filename)
        f = open(data_file, 'w')
        f.write('SEEDS Experiment Information:\n\n')
        f.write('Date and Time (UTC): %s\n' % (datetime.datetime.utcnow()))
        f.write('Experiment UUID: %s\n' % (self.experiment.uuid))
        f.write('Configuration File: %s\n' % (self.experiment.config.filename))

        sha256_checksum = hashlib.sha256()
        config_file = open(self.experiment.config.filename, 'rb')
        sha256_checksum.update(config_file.read())
        f.write('Configuration File SHA-256 Checksum: %s\n' % (sha256_checksum.hexdigest()))

        f.write('User Name: %s\n' % (os.getlogin()))
        f.write('Command Line: %s\n' % (sys.argv))
        f.write('Process ID: %d\n' % (os.getpid()))
        f.write('SEEDS Version: %s\n' % (s.__version__))
        f.write('NetworkX Version: %s\n' % (nx.__version__))
        f.write('Python Version: %s\n' % (sys.version))
        f.write('Platform: %s\n' % (sys.platform))
        f.write('Executable: %s\n' % (sys.executable))
        f.write('Exec Prefix: %s\n' % (sys.exec_prefix))
        f.write('Path: %s\n' % (sys.path))
        f.write('Modules: %s\n' % (sys.modules))
        f.write('Environment: %s\n' % (os.environ))
        f.close()




# -*- coding: utf-8 -*-
"""
Collection of commonly-used functions that deal with statistics, such as
calculating mean and standard deviation.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


def mean(data):
    """Calculate the mean of a list of numbers

    Parameters:

    *data*
        a list of numbers whose mean to calculate

    """

    return float(sum(data))/len(data)

def std(data):
    """Calculate the standard deviation of a list of numbers

    Parameters:

    *data*
        a list of numbers whose standard deviation to calculate

    """
    m = mean(data)
    sumsq = 0

    for d in data:
        sumsq += (d - m)**2

    return (sumsq / len(data))**(0.5)



# -*- coding: utf-8 -*-
"""
Manage plugins for the system

Plugins allow SEEDS to be extended to suit the needs of users without adding
to the base code and adding additional dependencies.  Examples include
additional Cells, Topologies, or Actions.  Plugins may be stored in multiple
directories.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import imp
import os
import sys

from seeds.Action import *
from seeds.Cell import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *
from seeds.Topology import *

class PluginManager(object):
    """ The PluginManager object keeps track of plugins in the specified plugin
    directories and allows users to query these plugins and receive references
    to the resulting objects.

    Properties:

    experiment
        A reference to the Experiment
    plugin_dirs
        A list of directories that may contain plugins
    plugins
        A module object containing attributes for each loaded plugin
        

    Configuration:
    
    The directories in which plugins can be searched for are specified by the
    plugin_dirs parameter in the [Experiment] block as a comma-separated list.

    Example:

    [Experiment]
    plugin_dirs = plugins,/opt/seedsplugins,../old_plugins

    """

    def __init__(self, experiment):
        """Initialize a PluginManager object

        Parameters:

        *experiment*
            A reference to the Experiment
        """

        self.experiment = experiment
        self.plugin_dirs = []
        self.plugins = None

        plugindirs = self.experiment.config.get(section=self.experiment.config_section, name='plugin_dirs')
        if plugindirs:
            for d in plugindirs.split(','):
                if os.path.exists(d):
                    self.append_dir(d)

    def load_plugins(self):
        """Scan the plugin directories and load the plugins"""

        self.plugins = None

        for plugindir in self.plugin_dirs:
            if os.path.exists(plugindir) and os.path.isdir(plugindir):
                for f in os.listdir(plugindir):
                    basename, extension = os.path.splitext(f)
                    tgt = os.path.join(plugindir, f)

                    if extension == ".py":
                        self.plugins = imp.load_source("obj", tgt)
                    elif extension == ".pyc":
                        self.plugins = imp.load_compiled("obj", tgt)

    def prepend_dir(self, dir):
        """Prepend a directory to the list of plugin directories.  After running,
        load_plugins() is called to maintain an up-to-datee list of plugins

        """

        if os.path.exists(dir):
            if self.plugin_dirs.count(dir) == 0:
                self.plugin_dirs.insert(0, dir)
                self.load_plugins()

    def append_dir(self, dir):
        """Append a directory to the list of plugin directories.  After running,
        load_plugins() is called to maintain an up-to-datee list of plugins

        """

        if os.path.exists(dir):
            if self.plugin_dirs.count(dir) == 0:
                self.plugin_dirs.append(dir)
                self.load_plugins()

    def get_plugin(self, plugin="", type=None):
        """Get a reference to the plugin.  The result may be then used to
        create new instances of that class or be executed as a function.

        Parameters:

        *plugin*
            The name of the plugin to get a reference for
        *type*
            The type of plugin to load (e.g., Cell, Action, ...).  If type is
            specified, matching plugins must be of the specified type.
            Otherwise, a PluginNotFoundError will be thrown.  If no type is
            specified, any plugin matching the given name will be acceptable.

        Example:
            obj_ref = get_plugin("MyObject")
            new_object = obj_ref()

        After this has executed, new_object will be an object of type MyObject.

        """

        try:
            ref = getattr(self.plugins, plugin)
        except AttributeError:
            raise PluginNotFoundError(plugin)
        else:
            if type != None and not issubclass(ref, type):
                raise PluginNotFoundError(plugin)
            return ref

    def plugin_exists(self, plugin=""):
        """Determine if a named plugin is present in the plugin directories"""
        return hasattr(self.plugins, plugin)



# -*- coding: utf-8 -*-
""" Print a number of measures related to the graphs used for topologies

Note that this action may take considerable time to execute for large
topologies.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import networkx as nx

import csv

from seeds.Action import *
from seeds.utils.statistics import mean, std

class PrintPopulationGraphProperties(Action):
    """ Write various properties of the population topology graph

    Configuration is done in the [PrintPopulationGraphProperties] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        The name of the file to write to (default:
        population_graph_properties.csv)
    header
        Whether or not to write a header to the output file.  The header will
        be an uncommented, comma-separated list of property names corresponding
        to the data in each row. (default: True)


    Configuration Example:

    [PrintPopulationGraphProperties]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = population_graph_properties.csv
    header = True

    """
    def __init__(self, experiment, label=None):
        """Initialize the PrintPopulationGraphProperties Action"""

        super(PrintPopulationGraphProperties, self).__init__(experiment,
                                                             name="PrintPopulationGraphProperties",
                                                             label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'population_graph_properties.csv')
        self.header = self.experiment.config.get(self.config_section, 'header', default=True)

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        if self.header:
            header = ['epoch', 'nodes', 'edges', 'avg_degree', 'std_degree',
                      'avg_clustering_coefficient','diameter',
                      'num_connected_components']
            self.writer.writerow(header)
      
    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        g = self.experiment.population.topology.graph
        degrees = list(nx.degree(g).values())
        row = [self.experiment.epoch, nx.number_of_nodes(g), nx.number_of_edges(g), mean(degrees), std(degrees), nx.average_clustering(g), nx.diameter(g), nx.number_connected_components(g)]
        self.writer.writerow(row)




# -*- coding: utf-8 -*-
"""
Collection of functions that perform different types of parsing
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import re


def parse_int_rangelist(s, sorted=False):
    """Parse a list of numeric ranges.  These lists are a comma-separated list
    of either single numbers or ranges, specified by number-number.

    Parameters:

    s
        A string containing a comma-separated list of integers and ranges of
        integers
    sorted
        Whether or not to sort the resulting list (default: False)

    """

    range_pattern = "\s*(\-?\d+)\s*\-\s*(\-?\d+)\s*"

    retval = []

    if s:
        tokens = s.split(",")
        for t in tokens:
            match = re.match(range_pattern, t)
            if match:
                start = int(match.group(1))
                end = int(match.group(2))
                for i in range(start, end+1):
                    retval.append(i)
            else:
                try:
                    x = int(t)
                    retval.append(x)
                except ValueError:
                    print("Error: Can not parse token '%s' from range list '%s'" % (t, s))

    if sorted:
        retval.sort()

    return retval



# -*- coding: utf-8 -*-
""" A population stores information related to the organisms that exist in an
experiment, including the graph that defines their interactions and a
dictionary to store any additional information about the population as a whole.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import random

from seeds.Experiment import *
from seeds.SEEDSError import *
from seeds.Topology import *
from seeds.utils.sampling import sample_with_replacement


class Population(object):
    """Manage populations of organisms

    Properties:

    experiment
        A reference to the Experiment in which this Population exists
    topology
        A graph representing the organisms (nodes) and the potential
        interactions between them (edges)
    label
        A unique label specifying the configuration of the Population
    config_section
        A string containing the section under which this population is
        configured
    _cell_class
        A reference to the proper class for the configured Cell type

    """

    def __init__(self, experiment, label=None):
        self.experiment = experiment
        self.label = label

        if self.label:
            self.config_section = "%s:%s" % ("Population", self.label)
        else:
            self.config_section = "%s" % ("Population")

        if not self.experiment.config.has_section(self.config_section):
            raise ConfigurationError("Configuration section %s not defined" % (self.config_section))

        self.experiment.data['population']['type_count'] = []
        self.experiment.data['population']['transitions'] = []

        # Create a topology to represent the organisms and their interactions
        pop_topology_raw = self.experiment.config.get(self.config_section, 'topology')
        parsed = pop_topology_raw.split(':')

        pop_topology_type = parsed[0]
        if len(parsed) > 1:
            label = parsed[1]
        else:
            label = None

        try:
            tref = self.experiment.plugin_manager.get_plugin(pop_topology_type,
                                                             type=Topology)
            self.topology = tref(self.experiment, label=label)
        except PluginNotFoundError as err:
            raise TopologyPluginNotFoundError(pop_topology_type)

        # Create a reference for the configured Cell type
        cell_config = self.experiment.config.get(self.config_section, 'cell')

        parsed = cell_config.split(':')
        cell_type = parsed[0]

        if len(parsed) > 1:
            label = parsed[1]
        else:
            label = None

        try:
            self._cell_class = self.experiment.plugin_manager.get_plugin(cell_type,
                                                                         type=Cell)
        except PluginNotFoundError as err:
            raise CellPluginNotFoundError(cell_type)

        # For each node in the topology, create a Cell and assign it the
        # coordinates of the node
        for n in self.topology.graph.nodes():
            c = self._cell_class(experiment=self.experiment, population=self,
                                 id=n, label=label)
            self.topology.graph.node[n]['cell'] = c

    def update(self):
        """Update the Population: update the topology stochastically

        During each update, a number of nodes in the topology are selected at
        random, and the update method of the Cells in the selected nodes is
        called.  By default, the number of nodes selected is equal to the
        number of nodes in the topology (so each node's Cell will be updated,
        on average, each epoch.  This number can be changed by setting the
        events_per_epoch parameter in the Experiment section of the
        configuration.
        
        """

        # Reset the transitions count.  Long-term transitions data can be
        # obtained by using the PrintCellTypeTransitions action
        num_types = self._cell_class.max_types
        self.experiment.data['population']['transitions'] = [[0]*num_types for i in range(num_types)]

        # Select a set of cells to update and update them
        events = self.experiment.config.getint(section=self.experiment.config_section,
                                               name='events_per_epoch',
                                               default=len(self.topology.graph))
        nodes_to_update = sample_with_replacement(self.topology.graph.nodes(), n=events)
        [self.topology.graph.node[n]['cell'].update() for n in nodes_to_update]

    def teardown(self):
        """Perform teardown at the end of an experiment"""
        self.topology.teardown()

    def increment_type_count(self, type):
        """Increment the cell type count for the given type

        Parameters:

        *type*
            The cell type whose count to increment

        """

        if len(self.experiment.data['population']['type_count']) <= type:
            self.experiment.data['population']['type_count'].extend([0] * (1 + type-len(self.experiment.data['population']['type_count'])))
        self.experiment.data['population']['type_count'][type] += 1

    def decrement_type_count(self, type):
        """Decrement the cell type count for the given type

        Parameters:

        *type*
            cell type whose count to decrement

        """

        self.experiment.data['population']['type_count'][type] -= 1

    def update_type_count(self, fromtype, totype):
        """Update the cell type counts, subtracting from the 'from' type and
        adding to the 'to' type

        Parameters:

        *fromtype*
            type that a cell was prior to being updated
        totype*
            type that a cell is after being updated

        """

        self.decrement_type_count(fromtype)
        self.increment_type_count(totype)
        self.add_transition(fromtype, totype)

    def add_transition(self, fromtype, totype):
        """Update the transition counts

        Parameters:

        *fromtype*
            type that a cell was prior to being updated
        totype*
            type that a cell is after being updated

        """

        self.experiment.data['population']['transitions'][fromtype][totype] += 1

    def cell_distance(self, src, dest):
        """Calculate the Cartesian distance between two cells

        Properties:

        src
            The first cell
        dest
            The second cell

        """

        return self.topology.node_distance(src.id, dest.id)

    def add_cell(self, cell=None, neighbors=[]):
        """Add a Cell of the appropriate type to the population and connect it
        to the given neighbors (optional).

        Parameters:

        *cell*
            An initialized Cell object to be added
        *neighbors*
            List of Cells to be connected to the newly-created Cell

        """

        neighbor_ids = []
        [neighbor_ids.append(n.id) for n in neighbors]

        new_id = max(self.topology.graph.nodes()) + 1

        try:
            self.topology.add_node(id=new_id, neighbors=neighbor_ids)
        except NonExistentNodeError as err:
            print("Error adding Cell: %s" % (err))

        if not Cell:
            cell = self._cell_class(experiment=self.experiment,
                                    population=self, id=new_id)

        self.topology.graph.node[n]['cell'] = cell

    def remove_cell(self, cell):
        """Remove the given Cell from the Population and its corresponding
        interactions

        Parameters:

        *cell*
            The Cell object to be removed

        """

        try:
            self.topology.remove_node(cell.id)
        except NonExistentNodeError as err:
            print("Error removing Cell: %s" % (err))

    def connect_cells(self, src, dest):
        """Connect two Cells in the Population

        This creates an edge in the topology between the corresponding nodes.
        These Cells can then be considered neighbors and may potentially
        interact with one another.

        Note that if the two Cells are already connected, this will not create
        an additional connection.

        Parameters:

        src
            The first Cell to connect
        dest
            The second Cell to connect

        """

        try:
            self.topology.add_edge(src.id, dest.id)
        except NonExistentNodeError as err:
            print("Error connecting Cells: %s" % (err))

    def disconnect_cells(self, src, dest):
        """Disonnect two Cells in the Population

        This removes the edge in the topology between the corresponding nodes.
        These Cells are then no longer considered neighbors and will not
        interact with one another.

        Parameters:

        src
            The first Cell to disconnect
        dest
            The second Cell to disconnect

        """

        try:
            self.topology.remove_edge(src.id, dest.id)
        except NonExistentEdgeError as err:
            print("Error disconnecting Cells: %s" % (err))




# -*- coding: utf-8 -*-
""" Lattice topology in which each node is connected with each of its 4
neighbors (von Neumann Neighborhood).  The radius of interactions can be
defined, which means all nodes within this many hops will be considered a
neighbor.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import networkx as nx

from seeds.SEEDSError import *
from seeds.Topology import *


class VonNeumannTopology(Topology):
    """
    Lattice topology with von Neumann Neighborhoods with configurable radius

    Nodes are organized on a lattice.  A node's neighbors reside to the left,
    right, above, below, and on the diagonals.  With radius 1, this neighborhood
    will contain 4 nodes.  With radius 2, a node will have 12.

    Configuration: All configuration options should be specified in the
    VonNeumannTopology block (unless otherwise specified by the config_section
    parameter).

        size
            Width/height, in number of nodes, of the Experiment.  With size 10,
            there would be 100 nodes. (no default)
        periodic
            Whether or not the Experiment should form a torus.  This means that
            nodes on the left border are neighbors with nodes on the right
            border. (default: False)
        radius
            Number of hops within a focal node's neighborhood (default: 1)

    Example:
        [VonNeumannTopology]
        size = 100
        periodic = True
        radius = 4

    """
    def __init__(self, experiment, config_section='VonNeumannTopology'):
        """Initialize a VonNeumannTopology object"""
        super(VonNeumannTopology, self).__init__(experiment, config_section=config_section)

        self.size = self.experiment.config.getint(self.config_section, 'size')
        self.periodic = self.experiment.config.getboolean(self.config_section, 'periodic', default=False)
        self.radius = self.experiment.config.getint(self.config_section, 'radius', default=1)

        if not self.size:
            raise ConfigurationError("VonNeumannTopology: size parameter must be defined")
        elif self.size < 1:
            raise ConfigurationError("VonNeumannTopology: size must be positive")
        elif self.radius < 0:
            raise ConfigurationError("VonNeumannTopology: radius must b greater than or equal to 0")
        elif self.radius >= self.size:
            raise ConfigurationError("VonNeumannTopology: radius can not exceed grid size")

        self.graph = self.vonneumann_2d_graph(self.size, self.size,
                                              radius=self.radius,
                                              periodic=self.periodic)

        for n in self.graph.nodes():
            self.graph.node[n]['coords'] = (self.row(n)/float(self.size), self.column(n)/float(self.size))

    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'Von Neumann Topology (%d nodes, %d radius)' % (self.size * self.size, self.radius)

    def row(self, nodeid):
        """Get the number of the row in which the given node is located

        Parameters:

        *nodeid*
            The ID of the node in question

        """
        return (nodeid/self.size)

    def column(self, nodeid):
        """Get the number of the column in which the given node is located

        Parameters:

        *nodeid*
            The ID of the node in question

        """
        return nodeid % self.size

    def node_id(self, row, col):
        """Get the ID of the node at the given row and column

        Parameters:

        *row*
            The row in which the node is located
        *col*
            The column in which the node is located

        """
        return row * self.size + col

    def vonneumann_2d_graph(self, rows=0, columns=0, radius=0,
                            periodic=False):
        """ Return the 2d grid graph of rows x columns nodes, each connected to
        its nearest Von Neumann neighbors within a given radius.  Optional
        argument periodic=True will connect boundary nodes via periodic
        boundary conditions.

        Parameters:

        *rows*
            The number of rows to be in the graph
        *columns*
            The number of columns to be in the graph
        *radius*
            The radius of interactions in the graph (there will be an edge
            between a node and all other nodes within N hops)
        *periodic*
            Prevent edge effects using periodic boundaries

        """
        
        if radius == 0:
            G = nx.empty_graph()
            G.add_edges_from(rows * columns)
        else:
            H = nx.grid_graph(dim=[rows,columns], periodic=periodic)

            m = {}
            for r in range(rows):
                for c in range(columns):
                    m[(r,c)] = (r * columns) + c

            H = nx.relabel_nodes(H, m)
            G = H.copy()

            for i in range(radius - 1):
                for n in H.nodes():
                    for neighbor in H.neighbors(n):
                        for nn in H.neighbors(neighbor):
                            G.add_edge(n, nn)

        G.name = "vonneumann_2d_radius_graph"
        return G

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_node is not supported by VonNeumannTopology")
        return

    def remove_node(self, id):
        """Remove a node from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_node is not supported by VonNeumannTopology")
        return

    def add_edge(self, src, dest):
        """Add an edge to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_edge is not supported by VonNeumannTopology")
        return

    def remove_edge(self, src, dest):
        """Remove an edge from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_edge is not supported by VonNeumannTopology")
        return

    def get_nearest_node(self, coords, n=1):
        """Return a list of the node(s) located nearest the given coordinates

        Note that this overwrites the get_nearest_node method defined by
        Topology (due to its simplicity vs using a k-d Tree).  This specific
        implementation may be removed in future releases if the more complete
        version in Topology is found to be better.

        Parameters:

        coords
            Tuple defining the point in question.  The number of dimensions
            should match the number of dimensions of the topology.
        n
            The number of nearest neighbors to find

        """

        if not coords or len(coords) < 1:
            print("ERROR: Invalid coordinates for get_nearest_node")
            return
        elif n < 1:
            print("ERROR: Invalid number of neighbors for get_nearest_node")
            return

        cell_width = 1.0 / self.size

        nearest_x = floor(float(coords[0]) / cell_width) * cell_width
        nearest_y = floor(float(coords[1]) / cell_width) * cell_width

        nearest_col = int(floor(coords[0] / cell_width))
        nearest_row = int(floor(coords[1] / cell_width))
        nearest_node = self.node_id(nearest_row, nearest_col)

        nearest = [nearest_node]

        for i in range(n - 1):
            print("ERROR: get_nearest_node only supports 1 neighbor at the moment")
            # TODO: look at neighbors of calculated nearest point and expand out.  Append to nearest.

        return nearest



# -*- coding: utf-8 -*-
"""
Collection of functions that implement commonly-used sampling/selection
algorithms, such as roulette selection.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import itertools
import random


def roulette_select(items=[], fitnesses=[], n=1):
    """Perform a fitness-proportional selection using a roulette wheel

    Parameters:

    *items*
        A list of items from which to select
    *fitnesses*
        A list of fitnesses of the corresponding items.  Larger values
        represent larger fitnesses and larger probabilities of being selected.
    *n*
        Number of items to select (default: 1)

    """

    if len(items) != len(fitnesses):
        print("Error: Must supply items")
    elif len(items) < 1:
        print("Error: Must supply items to choose from")

    total_fitness = float(sum(fitnesses))
    rel_fitnesses = [f/total_fitness for f in fitnesses]
    p = [sum(rel_fitnesses[:i+1]) for i in range(len(rel_fitnesses))]

    winners = []
    while len(winners) < n:
        for i in range(len(items)):
            if random.random() < p[i]:
                winners.append(items[i])
                break

    return winners

def sample_with_replacement(items=[], n=1):
    """Get a list of samples from a given set of items with replacement.

    Parameters:

    *items*
        A list of items from which to select
    *n*
        Number of items to select (default: 1)

    """

    samples = []
    popsize = len(items)

    if popsize < 1:
        print("Error: Must supply items")

    if n < 1:
        print("Error: Invalid number of samples")

    _random, _int = random.random, int
    samples = [items[_int(_random() * popsize)] for i in itertools.repeat(None, n)]

    return samples



# -*- coding: utf-8 -*-

""" The SetResourceAvailability action allows a resource to be made available
or unavailable.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.Action import *
from seeds.SEEDSError import *


class SetResourceAvailability(Action):
    """ Action to set the availability of a resource

    Configuration is done in the [SetResourceAvailability] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    resource
        The name of the resource whose availability to set
    available
        Whether or not the resource is available.  Valid settings are
        'available', 'on', '1', 'yes', 'true', 'unavailable', 'off', '0', 'no',
        or 'false'.


    Configuration Example:

    [SetResourceAvailability]
    resource = glucose
    available = yes
    epoch_start = 0
    epoch_end = 100
    frequency = 1
    priority = 0

    """

    def __init__(self, experiment, label=None):
        """Initialize the SetResourceAvailability Action"""

        super(SetResourceAvailability, self).__init__(experiment,
                                                name="SetResourceAvailability",
                                                label=label)

        self.threshold = self.experiment.config.getint(self.config_section, 'threshold', 0)
        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)

        self.resource = self.experiment.config.get(self.config_section, 'resource')

        try:
            self.res = self.experiment.resources[self.resource]
        except KeyError:
            raise ConfigurationError("SetResourceAvailability: Resource '%s' is undefined" % (self.resource))

        self.available = self.experiment.config.get(self.config_section, 'available')

        if str(self.available).lower() in ['available', 'on', '1', 'yes', 'true']:
            self.val = True
        elif str(self.available).lower() in ['unavailable', 'off', '0', 'no', 'false']:
            self.val = False
        else:
            raise ConfigurationError("SetResourceAvailability: Invalid value for availability '%s'" % (self.available))


    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        self.res.available = self.val



# -*- coding: utf-8 -*-

""" In SEEDS, environmental Resources are comprised of a graph of nodes, each
of which contains one ResourceCell object.  ResourceCells define the way in
which the defined resource behaves at that point in space.

TODO: more understandable description

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


class ResourceCell(object):
    """Interface for ResourceCell objects.  A ResourceCell object represents a
    Resource at a particular location in space.

    Properties:
    
    *config_section*
        The name of the section in the configuration file where parameter
        values are set
    *experiment*
        A reference to the experiment being run
    *id*
        A unique ID for this ResourceCell object
    *level*
        The level of the Resource at this point
    *resource*
        A reference to the Resource to which this ResourceCell belongs

    """

    def __init__(self, experiment, resource, config_section, id):
        """Initialize the ResourceCell object"""
        self.experiment = experiment
        self.resource = resource
        self.level = 0.0
        self.config_section = config_section
        self.id = id

    def __str__(self):
        """Return a string for when a ResourceCell object is printed"""
        return 'ResourceCell Object (Level: %f)' % (self.level)

    # By default, comparisons between two ResourceCell objects will be done
    # solely using their levels
    __lt__ = lambda self, other: self.level < other.level
    __le__ = lambda self, other: self.level <= other.level
    __eq__ = lambda self, other: self.level == other.level
    __ne__ = lambda self, other: self.level != other.level
    __gt__ = lambda self, other: self.level > other.level
    __ge__ = lambda self, other: self.level >= other.level

    def update(self):
        """Update the ResourceCell object in the node"""
        pass

    def teardown(self):
        """Perform any necessary cleanup at the end of the experiment"""
        pass

    def get_neighbors(self):
        """Get a list of neighboring ResourceCells"""
        return [self.resource.topology.graph.node[n]['resource'] for n in self.resource.topology.get_neighbors(self.id)]

    def coords(self):
        """Get the coordinates of the ResourceCell in space"""
        return self.resource.topology.graph.node[self.id]['coords']

    def get_neighbor_distance(self, neighbor):
        """Get the Cartesian distance to the given neighbor ResourceCell"""
        return self.resource.resourcetype_distance(self, neighbor)

    def get_neighbor_distances(self):
        """Get an array of distances to all neighbors"""
        return [self.get_neighbor_distance(n) for n in self.get_neighbors()]




# -*- coding: utf-8 -*-
"""
Collection of commonly-used functions that deal with geometry, such as
calculating distances.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.utils.numeric import is_numeric

def minkowski_distance_p(point1, point2, p=2, periodic=False):
    """Calculate the Minkowski distance to the pth power between two points.

    This is the Minkowski distance calculation without the root.

    Arguments:

    *point1*
        A tuple containing the coordinates of the first point
    *point2*
        A tuple containing the coordinates of the second point
    *p*
        Order parameter.  A value of 2 yields Euclidean distance, while a value
        of 1 yields Manhattan distance.  (Default: 2)
    *periodic*
        Whether or not periodic boundaries are used.  If they are, the distance
        along any dimension is the minimum of the distance between the two
        points not using the periodic edges or the distance using those edges.

    """
    dist = -1

    if len(point1) != len(point2):
        print("Error: dimensions do not match")
        return
    elif len(point1) > 1:
        dist = 0

        for dim in range(len(point1)):
            if periodic:
                d = abs(point1[dim] - point2[dim])
                d_periodic = abs(1-d)
                dist += min(d, d_periodic)**p
            else:
                dist += (point1[dim] - point2[dim])**p

    elif is_numeric(point1) and is_numeric(point2):
        if periodic:
            d = abs(point1 - point2)
            d_periodic = abs(1-d)
            dist = min(d, d_periodic)**p
        else:
            dist = (point1 - point2)**p

    return dist

def minkowski_distance(point1, point2, p=2, periodic=False):
    """Calculate the Minkowski distance between two points.

    Arguments:

    *point1*
        A tuple containing the coordinates of the first point
    *point2*
        A tuple containing the coordinates of the second point
    *p*
        Order parameter.  A value of 2 yields Euclidean distance, while a value
        of 1 yields Manhattan distance.  (Default: 2)
    *periodic*
        Whether or not periodic boundaries are used.  If they are, the distance
        along any dimension is the minimum of the distance between the two
        points not using the periodic edges or the distance using those edges.

    """

    return minkowski_distance_p(point1, point2, p=p, periodic=periodic)**(1.0/p)

def euclidean_distance(p1, p2, periodic=False):
    """Calculate the Euclidean distance between two points.

    Arguments:

    *p1*
        A tuple containing the coordinates of the first point
    *p2*
        A tuple containing the coordinates of the second point
    *periodic*
        Whether or not periodic boundaries are used.  If they are, the distance
        along any dimension is the minimum of the distance between the two
        points not using the periodic edges or the distance using those edges.

    """

    return minkowski_distance(point1=p1, point2=p2, p=2, periodic=periodic)

def euclidean_distance_squared(p1, p2, periodic=False):
    """Calculate the squared Euclidean distance between two points.  This
    function is intended to be used in cases where the sqrt in Euclidean
    distance can be skipped in order to speed up calculations.

    Arguments:

    *p1*
        A tuple containing the coordinates of the first point
    *p2*
        A tuple containing the coordinates of the second point
    *periodic*
        Whether or not periodic boundaries are used.  If they are, the distance
        along any dimension is the minimum of the distance between the two
        points not using the periodic edges or the distance using those edges.

    """

    return minkowski_distance_p(point1=p1, point2=p2, p=2, periodic=periodic)

def manhattan_distance(p1, p2, periodic=False):
    """Calculate the Manhattan distance between two points.

    Arguments:

    *p1*
        A tuple containing the coordinates of the first point
    *p2*
        A tuple containing the coordinates of the second point
    *periodic*
        Whether or not periodic boundaries are used.  If they are, the distance
        along any dimension is the minimum of the distance between the two
        points not using the periodic edges or the distance using those edges.

    """

    return minkowski_distance(point1=p1, point2=p2, p=1, periodic=periodic)




# -*- coding: utf-8 -*-
VERSION = (1, 0, 10)
__version__ = ".".join(map(str, VERSION[0:3])) + "".join(VERSION[3:])
__license__ = "Apache Version 2"
__download_url__ = "https://github.com/downloads/briandconnelly/seeds/seeds-%s.tar.gz" % (__version__)

from seeds.Action import *
from seeds.Cell import *
from seeds.Config import *
from seeds.Experiment import *
from seeds.PluginManager import *
from seeds.Population import *
from seeds.Resource import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *
from seeds.Topology import *




# -*- coding: utf-8 -*-
"""
SineResource represents a resource whose levels fluctuate sinusoidally between
0 and two times the configured amplitude (i.e., the wave is shifted up one
amplitude).
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from math import sin, pi

from seeds.Action import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *
from seeds.utils.parsing import parse_int_rangelist


class SineResource(ResourceCell):
    """Environmental Resource class

    Properties:

    experiment
        A reference to the experiment being run
    resource
        A reference to the Resource to which this ResourceCell belongs
    config_section
        The name of the section in the configuration file where parameter
        values are set
    id
        A unique ID for this ResourceCell object
    name
        Unique name of the resource
    amplitude
        Defines the high value for the resource.  Minimum value will be 0, and
        maximum value will be 2*amplitude (default: 0.0).
    period
        Period the length of time (epochs) required to complete one cycle
        (integer values, units: epochs, default: 1).
    phase
        Phase specifies where in the cycle the resource level should be at
        epoch 0 (units: epochs, default: 0).

    SineResource Resources can be defined in config files using a
    [Resource:<uniquename>] section and type SineResource.  For example:

        [Resource:sun]
        type = SineResource
        amplitude = 3.2
        period = 100
        phase = 5

    """
    def __init__(self, experiment, resource, config_section, id):
        """ Initialize a SineResource object

        Parameters:

        *experiment*
            A pointer to the Experiment
        *resource*
            A pointer to the Resource of which this is a part
        *config_section*
            The name under which the configuration parameters are specified for
            this Resource
        *id*
            A unique ID for this node in the resource graph

        """
        super(SineResource, self).__init__(experiment=experiment,
                                           resource=resource,
                                           config_section=config_section,
                                           id=id)

        self.amplitude = self.experiment.config.getfloat(self.config_section, "amplitude", default=0.0)
        self.period = self.experiment.config.getint(self.config_section, "period", default=0)
        self.phase = self.experiment.config.getint(self.config_section, "phase", default=0)

        if self.amplitude < 0:
            raise ConfigurationError("SineResource: amplitude for '%s' must be at least 0" % (self.resource.name))
        elif self.period <= 0:
            raise ConfigurationError("SineResource: period for '%s' must be greater than 0" % (self.resource.name))
        elif self.phase < 0:
            # BDC: really, a negative phase should be ok...
            raise ConfigurationError("SineResource: phase for '%s' must be greater than 0" % (self.resource.name))

        # Set the initial level
        self.update()
        
    def __str__(self):
        """Produce a string to be used when a SineResource object is printed"""
        return "SineResource [Name: %s][Level: %f][Amplitude: %f][Period: %d][Phase: %d]" % (self.name, self.level, self.amplitude, self.period, self.phase)

    def set_amplitude(self, value):
        """Set the amplitude of the resource

        Parameters:

        *value*
            The new amplitude of the resoruce

        """
        self.amplitude = value

    def set_period(self, value):
        """Set the period of the resource

        Parameters:

        *value*
            The new period of the resoruce

        """
        self.period = value

    def set_phase(self, value):
        """Set the phase of the resource

        Parameters:

        *value*
            The new phase of the resoruce

        """
        self.phase = value

    def update(self):
        """Update the level of the resource.  Even if the resource is
        unavailable, the resource level will be updated.
        
        """

        position_radians = ((self.experiment.epoch * 1.0) / self.period) * 2 * pi
        phase_radians = ((self.phase * 1.0) / self.period) * 2 * pi
        self.level = (self.amplitude * sin(position_radians + phase_radians)) + self.amplitude
        self.experiment.data['resources'][self.resource.name]['levels'][self.id] = self.level


class SetSineResourceProperties(Action):
    """ Action to set the properties (period, high, low, or duty cycle) of a
    SineResource resource

    Configuration is done in the [SetSineResourceProperties] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    resource
        The name of the resource whose availability to set
    cells
        A list of the ResourceCell cells whose inflow to set.  If none are
        specified, the inflows all cells will be set.
    period
        Period is the length of time (epochs) required to complete one cycle.
        If no value is specified, the period will not be altered.  (integer
        values, units: epochs)
    amplitude
        Amplitude specifies the amplitude of the resource.  If no value is
        specified, the high value will not be altered.

    Configuration Example:

    [SetSineResourceProperties]
    resource = glucose
    period = 100
    epoch_start = 0
    epoch_end = 100
    frequency = 1
    priority = 0

    """

    def __init__(self, experiment, label=None):
        """Initialize the SetSineResourceProperties Action"""

        super(SetSineResourceProperties, self).__init__(experiment,
                                                name="SetSineResourceProperties",
                                                label=label)

        self.threshold = self.experiment.config.getint(self.config_section, 'threshold', 0)
        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)

        self.resource = self.experiment.config.get(self.config_section, 'resource')

        try:
            self.res = self.experiment.resources[self.resource]
        except KeyError:
            raise ConfigurationError("SetSinSinerceProperties: Resource '%s' is undefined" % (self.resource))

        if self.res.type != 'SineResource':
            raise ConfigurationError("SetSineResourceProperties: Resource '%s' is not a SineResource" % (self.resource))
            

        self.period = self.experiment.config.getfloat(self.config_section, 'period')
        if self.period and self.period <= 0:
            raise ConfigurationError("SetSineResourceProperties: Invalid value for period '%f'.  Must be nonzero and nonnegative." % (self.period))

        self.amplitude = self.experiment.config.getfloat(self.config_section, 'amplitude')
        if self.amplitude and self.amplitude < 0:
            raise ConfigurationError("SetSineResourceProperties: Invalid value for amplitude '%f'.  Must be nonnegative." % (self.amplitude))

        self.cells_str = self.experiment.config.get(self.config_section, 'cells')

        if not self.cells_str:
            self.cells = self.res.topology.graph.nodes()
        else:
            self.cells = parse_int_rangelist(self.cells_str, sorted=True)
            nids = self.res.topology.graph.nodes()
            for c in self.cells:
                if c not in nids:
                    raise ConfigurationError("SetSineResourceProperties: Cell %d does not exist in Resource '%s'" % (c, self.resource))

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        for c in self.cells:
            if self.period:
                self.res.topology.graph.node[c]['resource'].period = self.period
            if self.amplitude:
                self.res.topology.graph.node[c]['resource'].amplitude = self.amplitude



# -*- coding: utf-8 -*-
"""
SquareResource represents a resource whose levels fluctuate between two
specific values.  These values and the fraction of time spent in either state
can be configured.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.Action import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *
from seeds.utils.parsing import parse_int_rangelist


class SquareResource(ResourceCell):
    """Environmental Resource class

    Properties:

    id
        A unique ID for this node in the resource graph
    experiment
        A reference to the Experiment being performed
    resource
        A reference to the Resource of which this object is a part
    level
        The current level of the resource
    config_section
        The name of the section where parameters for this object are defined
    period
        period is the length of time (epochs) required to complete one cycle
        (integer values, units: epochs, default: 0).
    high
        high specifies the maximum value of the resource, which occurs in the
        high state (default: 0.0).
    low
        low specifies the minimum value of the resource, which occurs in the
        low state (default: 0.0).
    duty_cycle
        duty cycle is the fraction of one period that a resource spends in high
        state (default: 0.5).
    offset
        Offset defines the amount of time in epochs before the first high state
        is encountered (default: 0).

    SquareResource Resources can be defined in config files using a
    [Resource:<uniquename>] section and SquareResource type.  For example:

        [Resource:enzyme]
        type = SquareResource
        topology = CartesianTopology
        period = 100
        high = 5.8
        low = 0.0
        duty_cycle = 0.33
        offset = 5

    """

    def __init__(self, experiment, resource, config_section, id):
        """ Initialize a SquareResource object

        Parameters:

        *experiment*
            A pointer to the Experiment
        *resource*
            A pointer to the Resource of which this is a part
        *config_section*
            The name under which the configuration parameters are specified for
            this Resource
        *id*
            A unique ID for this node in the resource graph

        """
        super(SquareResource, self).__init__(experiment=experiment,
                                             resource=resource,
                                             config_section=config_section,
                                             id=id)

        self.period = self.experiment.config.getint(self.config_section, "period", default=0)
        self.high = self.experiment.config.getfloat(self.config_section, "high", default=0.0)
        self.low = self.experiment.config.getfloat(self.config_section, "low", default=0.0)
        self.duty_cycle = self.experiment.config.getfloat(self.config_section, "duty_cycle", default=0.5)
        self.offset = self.experiment.config.getint(self.config_section, "offset", default=0)

        if self.period <= 0:
            raise ConfigurationError("SqureResource: period for '%s' must be greater than 0" % (self.resource.name))
        elif self.high < self.low:
            raise ConfigurationError("SqureResource: high vale for '%s' must be greater than low value" % (self.resource.name))
        elif self.duty_cycle < 0:
            raise ConfigurationError("SqureResource: duty cycle for '%s' must at least 0" % (self.resource.name))
        elif self.duty_cycle > 1:
            raise ConfigurationError("SqureResource: duty cycle for '%s' must be less than 1" % (self.resource.name))
        elif self.offset < 0:
            raise ConfigurationError("SqureResource: offset cycle for '%s' must at least 0" % (self.resource.name))

        # Set the initial level
        self.update()
        
    def __str__(self):
        """Produce a string to be used when a SquareResource object is printed"""
        return "SquareResource [Name: %s][Level: %f][Offset: %f][Period: %d][High: %f][Low: %f][Duty Cycle: %f]" % (self.name, self.level, self.offset, self.high, self.low, self.duty_cycle)

    def set_offset(self, value):
        """Set the offset of the resource

        Parameters:

        *value*
            The new offset of the resoruce

        """

        self.offset = value

    def set_period(self, value):
        """Set the period of the resource

        Parameters:

        *value*
            The new period of the resoruce

        """

        self.period = value

    def set_high(self, value):
        """Set the high of the resource

        Parameters:

        *value*
            The new high of the resoruce

        """

        self.high = value

    def set_low(self, value):
        """Set the low of the resource

        Parameters:

        *value*
            The new low of the resoruce

        """

        self.low = value

    def set_duty_cycle(self, value):
        """Set the duty_cycle of the resource

        Parameters:

        *value*
            The new duty_cycle of the resoruce

        """

        self.duty_cycle = value


    def update(self):
        """Update the level of the resource.  Even if the resource is
        unavailable, the resource level will be updated.
        
        """

        position = ((self.experiment.epoch - self.offset) % (self.period)) / (self.period * 1.0)

        if position < self.duty_cycle:
            self.level = self.high
        else:
            self.level = self.low

        self.experiment.data['resources'][self.resource.name]['levels'][self.id] = self.level


class SetSquareResourceProperties(Action):
    """ Action to set the properties (period, high, low, or duty cycle) of a
    SquareResource resource

    Configuration is done in the [SetSquareResourceProperties] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    resource
        The name of the resource whose availability to set
    cells
        A list of the ResourceCell cells whose inflow to set.  If none are
        specified, the inflows all cells will be set.
    period
        Period is the length of time (epochs) required to complete one cycle.
        If no value is specified, the period will not be altered.  (integer
        values, units: epochs)
    high
        High specifies the maximum value of the resource, which occurs in the
        high state.  If no value is specified, the high value will not be
        altered.
    low
        Low specifies the minimum value of the resource, which occurs in the
        low state.  If no value is specified, the low value will not be
        altered.
    duty_cycle
        Duty cycle is the fraction of one period that a resource spends in high
        state.  If no value is specified, the duty cycle will not be altered.

    Configuration Example:

    [SetSquareResourceProperties]
    resource = glucose
    high = 10.2
    period = 100
    epoch_start = 0
    epoch_end = 100
    frequency = 1
    priority = 0

    """

    def __init__(self, experiment, label=None):
        """Initialize the SetSquareResourceProperties Action"""

        super(SetSquareResourceProperties, self).__init__(experiment,
                                                name="SetSquareResourceProperties",
                                                label=label)

        self.threshold = self.experiment.config.getint(self.config_section, 'threshold', 0)
        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)

        self.resource = self.experiment.config.get(self.config_section, 'resource')

        try:
            self.res = self.experiment.resources[self.resource]
        except KeyError:
            raise ConfigurationError("SetSquareResourceProperties: Resource '%s' is undefined" % (self.resource))

        if self.res.type != 'SquareResource':
            raise ConfigurationError("SetSquareResourceProperties: Resource '%s' is not a SquareResource" % (self.resource))
            

        self.period = self.experiment.config.getfloat(self.config_section, 'period')
        if self.period and self.period < 0:
            raise ConfigurationError("SetSquareResourceProperties: Invalid value for period '%f'.  Must be nonnegative." % (self.period))


        self.high = self.experiment.config.getfloat(self.config_section, 'high')
        self.low = self.experiment.config.getfloat(self.config_section, 'low')


        if self.high and self.low and self.high < self.low:
            raise ConfigurationError("SetSquareResourceProperties: High value must not be greater than low value." % (self.period))

        # NOTE: should also make sure that new high or low values (if one is not specified) is valid with the current configuration

        self.duty_cycle = self.experiment.config.getfloat(self.config_section, 'duty_cycle')
        if self.duty_cycle and self.duty_cycle < 0:
            raise ConfigurationError("SetSquareResourceProperties: Invalid value for duty_cycle '%f'.  Must be nonnegative." % (self.duty_cycle))

        if not self.period and not self.high and not self.low and not self.duty_cycle:
            raise ConfigurationError("SetSquareResourceProperties: Must specify value for period, high, low, or duty_cycle")

        self.cells_str = self.experiment.config.get(self.config_section, 'cells')

        if not self.cells_str:
            self.cells = self.res.topology.graph.nodes()
        else:
            self.cells = parse_int_rangelist(self.cells_str, sorted=True)
            nids = self.res.topology.graph.nodes()
            for c in self.cells:
                if c not in nids:
                    raise ConfigurationError("SetSquareResourceProperties: Cell %d does not exist in Resource '%s'" % (c, self.resource))

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        for c in self.cells:
            if self.low:
                self.res.topology.graph.node[c]['resource'].low = self.low
            if self.high:
                self.res.topology.graph.node[c]['resource'].high = self.high
            if self.duty_cycle:
                self.res.topology.graph.node[c]['resource'].duty_cycle = self.duty_cycle
            if self.period:
                self.res.topology.graph.node[c]['resource'].period = self.period



# -*- coding: utf-8 -*-
""" A Resource object represents the amount of a particular resource at a given
location in space.  Resources have a level that can be checked, consumed, or
added to.  Different Resource objects may have different objects such as inflow
and may interact with neighboring Resource objects (e.g., for diffusion).

This class provides the base properties and functionality for all Resources
used in SEEDS.  Specific implementations can be seen in plugins/resource.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.Experiment import *
from seeds.PluginManager import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *
from seeds.Topology import *
from seeds.utils.sampling import sample_with_replacement


class Resource(object):
    """Interface for Resources

    Properties:

    available
        Whether or not the resource is currently available (default: True)
    experiment
        A reference to the Experiment in which the Resource exists
    name
        Unique name of the resource
    type
        The type of the resource (specific ResourceCell class to be used)
    topology
        The Topology object that stores the graph of ResourceCell (nodes)
        objects and the flow between them (edges)
    _resource_type_class
        A reference to the proper class for the configured ResourceCell


    Resources are defined in config files using a [Resource:<label>]
    section.  For example:

    [Resource:glucosev1}
    name = glucose
    type = NormalResource
    initial = 10.0
    inflow = 1
    outflow = 0.1
    decay = 0.1

    For more information about the properties of this resource, see the
    documentation for NormalResource.

    """

    def __init__(self, experiment, label=None):
        """ Initialize a Resource object

        Parameters:

        *experiment*
            A pointer to the Experiment
        *label*
            A label for a unique configuration of a resource.

        """

        self.experiment = experiment

        if label:
            self.label = label
        else:
            raise ConfigurationError("Must supply a Resource label")

        self.config_section = "Resource:%s" % (self.label)
        self.name = self.experiment.config.get(self.config_section, "name",
                                               default=self.label)

        self.experiment.data['resources'][self.name] = {}

        self.type = self.experiment.config.get(self.config_section, 'type',
                                               default='NormalResource')
        self.available = self.experiment.config.getboolean(self.config_section,
                                                           'available',
                                                           default=True)

        try:
            self._resource_type_class = self.experiment.plugin_manager.get_plugin(self.type, type=ResourceCell)
        except PluginNotFoundError as err:
            raise ResourceCellPluginNotFoundError(self.type)

        topology_raw = self.experiment.config.get(self.config_section, 'topology')
        parsed = topology_raw.split(':')

        topology_type = parsed[0]

        if len(parsed) > 1:
            top_label = parsed[1]
        else:
            top_label = None

        if topology_type != "MooreTopology" and topology_type != "VonNeumannTopology":
            raise ConfigurationError("SEEDS does not currently support Resource topology types other than MooreTopology or VonNeumannTopology")

        try:
            tref = self.experiment.plugin_manager.get_plugin(topology_type, type=Topology)
        except PluginNotFoundError as err:
            raise TopologyPluginNotFoundError(topology_type)

        self.topology = tref(experiment=self.experiment,
                             label=top_label)

        self.experiment.data['resources'][self.name]['levels'] = [0] * self.topology.num_nodes()

        # For each node in the topology, create a ResourceCell object
        for n in self.topology.graph.nodes():
            self.topology.graph.node[n]['resource'] = self._resource_type_class(experiment=self.experiment,
                                                                                resource=self, 
                                                                                config_section=self.config_section,
                                                                                id=n)

    def __str__(self):
        """Produce a string to be used when a Resource object is printed"""
        return "Resource [Name: %s][Topology: %s]" % (self.name, self.topology)

    def update(self):
        """Update the Resource

        During each update, a number of nodes in the topology are selected at
        random, and the update method of the ResourceCell in the selected nodes
        is called.  By default, the number of nodes selected is equal to the
        number of nodes in the topology (so each node's ResourceCell will be
        updated, on average, each epoch.  This number can be changed by setting
        the events_per_epoch parameter in the Experiment section of the
        configuration.
                                                                        
        """

        events = self.experiment.config.getint(section=self.config_section,
                                               name='events_per_epoch',
                                               default=len(self.topology.graph))
        nodes_to_update = sample_with_replacement(self.topology.graph.nodes(), n=events)
        [self.topology.graph.node[n]['resource'].update() for n in nodes_to_update]

    def teardown(self):
        """Perform any necessary cleanup at the end of the experiment"""
        self.topology.teardown()

    def add_resourcetype(self, rt=None, neighbors=[]):
        """Add a ResourceCell of the appropriate type to the Resource and
        connect it to the given neighbors (optional).

        BDC: Does this method make sense?

        Parameters:

        *rt*
            An initialized ResourceCell object to be used.  If this argument is
            not supplied, one will be created.
        *neighbors*
            List of ResourceCells to be connected to the newly-created
            ResourceCell

        """

        neighbor_ids = []
        [neighbor_ids.append(n.id) for n in neighbors]

        new_id = max(self.topology.graph.nodes()) + 1

        try:
            self.topology.add_node(id=new_id, neighbors=neighbor_ids)
        except NonExistentNodeError as err:
            print("Error adding ResourceCell: %s" % (err))

        if not rt:
            rt = self._resource_type_class(experiment=self.experiment,
                                           resource=self,
                                           config_section=self.config_section,
                                           id=new_id)
 
        self.topology.graph.node[n]['resource'] = rt

    def remove_resourcetype(self, rt):
        """Remove the given ResourceCell from the Resource and its
        corresponding interactions

        Parameters:

        *rt*
            The ResourceCell object to be removed

        """

        try:
            self.topology.remove_node(cell.id)
        except NonExistentNodeError as err:
            print("Error removing ResourceCell: %s" % (err))

    def connect_resourcetypes(self, src, dest):
        """Connect two ResourceCell objects in the Population

        This creates an edge in the topology between the corresponding nodes.
        These ResourceCells can then be considered neighbors and may
        potentially interact with one another.

        Note that if the two ResourceCells are already connected, this will not
        create an additional connection.

        Parameters:

        src
            The first ResourceCell to connect
        dest
            The second ResourceCell to connect

        """

        try:
            self.topology.add_edge(src.id, dest.id)
        except NonExistentNodeError as err:
            print("Error connecting ResourceCells: %s" % (err))

    def disconnect_resourcetypes(self, src, dest):
        """Disonnect two ResourceCells in the Population

        This removes the edge in the topology between the corresponding nodes.
        These ResourceCells are then no longer considered neighbors and will
        not interact with one another.

        Parameters:

        src
            The first ResourceCell to disconnect
        dest
            The second ResourceCell to disconnect

        """

        try:
            self.topology.remove_edge(src.id, dest.id)
        except NonExistentEdgeError as err:
            print("Error disconnecting ResourceCells: %s" % (err))

    def resourcetpye_distance(self, src, dest):
        """Calculate the Cartesian distance between two ResourceCells

        Properties:

        src
            The first ResourceCell
        dest
            The second ResourceCell

        """

        return self.topology.node_distance(src.id, dest.id)




# -*- coding: utf-8 -*-

""" This template outlines the necessary components for implementing a new
Topology in SEEDS.  Topologies are used in SEEDS to model the interactions
between organisms and the flow of resources through space.  These are modeled
as graphs, which a Topology creates and maintains.  These graphs could be
simple lattices (as used by SEEDS' included MooreTopology and
VonNeumannTopology) or more complex structures.

Areas marked with 'TODO' should be replaced with code specific to the Topology
being implemented.

Generally, creating a new Topology requires creating a new constructor
(__init__), building a graph using NetworkX, and implementing methods to add
and remove nodes and edges.

The name of the Topology type should be the name of the class.  The name of the
file should also match this.

Methods and parameters common to all Topology objects can be seen in the
Topology.py file in the main SEEDS codebase.

Once completed, new Topology type files can be placed in a plugins directory
and used by specifying the name of the file/class with the "topology" parameter
in the [Population] or a [Resource]  section of the configuration file.

"""

__author__ = "TODO"
__credits__ = "TODO"


import networkx as nx
import random

from seeds.SEEDSError import *
from seeds.Topology import *


class TODO-TopologyName(Topology):
    """ TODO: documentation about this Topology type including the motivation
    behind it, any parameters that can be defined in configuration files, and
    other pertinent information.
    """

    # TODO: the __init__ method (or "constructor") is called when a new
    # Topology object is created.  Generally, a constructor will read parameter
    # values from a config file, validate those values, and build the graph.

    def __init__(self, experiment, config_section='TODO-TopologyName'):
        """Initialize a TODO-TopologyName object"""
        super(TODO-TopologyName, self).__init__(experiment, config_section=config_section)

        # TODO: here is a good place to get parameter values from the configuration file
        self.size = self.experiment.config.getint(self.config_section, 'size')

        # TODO: the constructor is a good place to make sure configured values are valid
        if self.size < 1:
            raise ConfigurationError("TODO-TopologyName: size must be positive")


        # TODO: create the graph.  SEEDS uses NetworkX for graph functionality,
        # so a number of predefined graph types can be used, or new ones can be
        # created by adding nodes and edges accordingly.  See
        # http://networkx.lanl.gov/.

        self.graph = TODO

        # TODO: it is also good to assign coordinates to each node in the
        # graph.  This tuple is placed in the ['coords'] attribute of each
        # node.  Here, we assign each node a random 2-dimensional location.
        for n in self.graph.nodes(): self.graph.node[n]['coords'] = (random.random(), random.random())


    # TODO: the __str__ method returns a string to be used when an object is
    # printed.  This can be a useful place to return information that is
    # specific to the Topology type
    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return 'TODO-TopologyName Topology (%d nodes)' % (self.size)


    # TODO: the following methods allow the underlying graph to be altered
    # through the addition and removal of nodes and edges.  The basic Topology
    # class provides these functions on its own, so if you wish to use them,
    # these methods can be removed from this file.  However, if you wish to use
    # a static graph, these methods should be overrided as below to prevent the
    # graph from being modified.
    
    # Below, instead of modifying the graph whenever these methods are called,
    # nothing is done.  In these cases, it is useful to either throw a
    # ConfigurationError or print an error message indicating that these
    # operations are not supported when these methods are called.

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph."""
        return

    def remove_node(self, id):
        """Remove a node from the graph."""
        return

    def add_edge(self, src, dest):
        """Add an edge to the graph."""
        return

    def remove_edge(self, src, dest):
        """Remove an edge from the graph."""
        return




# -*- coding: utf-8 -*-
""" Print basic statistics about the distribution of a Resource
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import csv

from seeds.Action import *
from seeds.SEEDSError import *
from seeds.utils.statistics import mean, std


class PrintResourceStats(Action):
    """ Write information about the distribution of the given resource

    Configuration is done in the [PrintResourceStats] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        The base name of the file to write to.  The name of the resource will
        be appended, so a filename of 'resource', when printing information
        about a resource named 'glucose', would produce the file
        'resource-glucost.csv'.  (default: resource)
    header
        Whether or not to write a header to the output file.  The header will
        be an uncommented, comma-separated list of property names corresponding
        to the data in each row. (default: True)
    resource
        The name of the resource about which to print information


    Configuration Example:

    [PrintResourceStats]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = resource
    header = True
    resource = glucose

    """

    def __init__(self, experiment, label=None):
        """Initialize the PrintResourceStats Action"""

        super(PrintResourceStats, self).__init__(experiment,
                                                 name="PrintResourceStats",
                                                 label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'resource')
        self.header = self.experiment.config.getboolean(self.config_section, 'header', default=True)
        self.resource = self.experiment.config.get(self.config_section, 'resource')

        try:
            self.res = self.experiment.resources[self.resource]
        except KeyError:
            raise ConfigurationError("PrintResourceStats: Resource '%s' is undefined" % (self.resource))

        full_filename = "%s-%s.csv" % (self.filename, self.resource)
        data_file = self.datafile_path(full_filename)
        self.writer = csv.writer(open(data_file, 'w'))

        if self.header:
            header = ['epoch', 'mean', 'standard_deviation', 'available']
            self.writer.writerow(header)

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        levels = self.experiment.data['resources'][self.resource]['levels']
        row = [self.experiment.epoch, mean(levels), std(levels), int(self.res.available)]
        self.writer.writerow(row)




# -*- coding: utf-8 -*-

""" This template outlines the necessary components for implementing a new
ResourceCell in SEEDS.  Generally, this involves implementing a constructor
(__init__) and an update method.  In SEEDS, Resources are modeled as graphs of
ResourceCell nodes, which define the Resource at a specific location in the
world.

Areas marked with 'TODO' should be replaced with code specific to the
ResourceCell being implemented.

The name of the ResourceCell type should be the name of the class.  The name of
the file should also match this.

Methods and parameters common to all ResourceCell objects can be seen in the
ResourceCell.py file in the main SEEDS codebase.

Once completed, new ResourceCell type files can be placed in a plugins
directory and used by specifying the name of the file/class with the "type"
parameter in the appropriate [Resource] section of the configuration file.

"""

__author__ = "TODO"
__credits__ = "TODO"

from seeds.Action import *
from seeds.ResourceCell import *
from seeds.SEEDSError import *


class TODO-ResourceCellName(ResourceCell):
    """ TODO: Documentation for this ResourceCell type including its
    motivations, how it is configured, and how it works.

    """

    # TODO: constructor for ResourceCell objects.
    def __init__(self, experiment, resource, config_section, id):
        """ Initialize a NormalResource object

        Parameters:

        *experiment*
            A pointer to the Experiment
        *resource*
            A pointer to the Resource of which this is a part
        *config_section*
            The name under which the configuration parameters are specified for
            this Resource
        *id*
            A unique ID for this node in the resource graph

        """
        super(NormalResource, self).__init__(experiment=experiment,
                                             resource=resource,
                                             config_section=config_section,
                                             id=id)

        # TODO: get configuration values.  Here, "inflow" is used for example purposes.
        self.inflow = self.experiment.config.getfloat(self.config_section, "inflow", default=0.0)

        # TODO: validate any configuration values
        if self.inflow < 0:
            raise ConfigurationError("NormalResource: inflow for '%s' can not be negative" % (self.resource.name))

        # TODO: initialize the ResourceCell
        self.level = 0.0

    # TODO: the __str__ method returns a string to be used when an object is
    # printed.  This can be a useful place to return information that is
    # specific to the ResourceCell type
    def __str__(self):
        """Produce a string to be used when an object is printed"""
        return "TODO-ResourceCellName [Name: %s][Level: %f]" % (self.name, self.level)


    # TODO: the update method updates the level of the resouce according to the
    # rules for this ResourceCell.

    def update(self):
        """Update the level of the resource.  Even if the resource is
        unavailable, the resource level will be updated.
        
        """

        # NOTE: If the state depends on neighbors, the list of neighbor
        # ResourceCells can be retrieved.
        neighbors = self.get_neighbors()

        # TODO: calculate the new level
        self.level = TODO

        # NOE: The levels of all ResourceCells are kept in a list.  This
        # enables information about the distribution of resources to be
        # retrieved without traversing the graph.  Here, we update the level
        # associated with this cell.
        self.experiment.data['resources'][self.resource.name]['levels'][self.id] = self.level


# TODO: typically, any Actions associated with this ResourceCell type are also
# defined in this file.  Refer to the Action template for more information.



# -*- coding: utf-8 -*-
""" Cell type that represents bacteriocin-producing bacteria.  In this system,
cells can be either toxin producers, toxin resistant, or toxin sensitive.
Toxin producers produce a toxin that kills nearby sensitive cells.  This toxin
does not effect resistant or producer cells.

The costs of toxin resistance and production each manifest themselves as
increases in mortality.  Therefore, producers have the highest death rate,
followed by resistant cells, and finally sensitive cells.

This is a non-transitive system that can be compared with the
rock-paper-scissors (RPS) game, where each strategy beats one but is beaten by
another.  If any one strategy is lost, another strategy will also be lost, as
the non-transitivity is broken (if no one can play 'rock', 'scissors' will
always win the game).

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly, Luis Zaman, Ben Kerr"

from seeds.Cell import *
import random

class Kerr07Cell(Cell):
    """
    This is a Cell type inspired by the model presented in:

        B. Kerr, Bacteriocins: ecology and evolution. Springer, 2007

    and used in:

        B.D. Connelly, L. Zaman, C. Ofria, and P.K. McKinley,
        "Social structure and the maintenance of biodiversity," in
        Proceedings of the 12th International Conference on the Synthesis
        and Simulation of Living Systems (ALIFE), pp. 461-468, 2010.

    This Cell type has 4 states:
        EMPTY: no organism is in the cell
        SENSITIVE: cells sensitive to being killed by bacteriocin
        RESISTANT: cells insensitive to bacteriocin, but that do not
          produce it themselves.
        PRODUCER: cells that produce bacteriocin and are insensitive
          to it.

    The sensitivity of a SENSITIVE cell to bacteriocin depends on the
    number of PRODUCER cells in its neighborhood.


    Configuration: All configuration options should be specified in a
    Kerr07Cell block.

        death_sensitive: death rate of SENSITIVE cells (float, [0,1])
        death_resistant: death rate of RESISTANT cells (float, [0,1])
        death_producer: death rate of PRODUCER cells (float, [0,1])
        toxicity: The toxicity of PRODUCER cells (float, [0,1])

    Example:
        [Kerr07Cell]
        death_sensitive = 0.250
        death_resistant = 0.312
        death_producer = 0.333
        toxicity = 0.650

    """

    types = ['Empty', 'Sensitive', 'Resistant', 'Producer']
    max_types = 4
    type_colors = ['#777777','b','g','r']

    EMPTY = 0
    SENSITIVE = 1
    RESISTANT = 2
    PRODUCER = 3

    def __init__(self, experiment, population, id, type=-1, name="Kerr07Cell", label=None):
        """Initialize a Kerr07Cell object

        The type for the cell is selected at random.

        Parameters:

        *experiment*
            A reference to the Experiment in which the Cell will reside
        *population*
            A reference to the Population in which this Cell resides
        *id*
            A unique ID for the cell
        *type*
            The type of cell to initialize (-1 for random)
        *name*
            The name of this Cell type
        *label*
            A unique label for configuring this Cell type

        """

        super(Kerr07Cell, self).__init__(experiment, population, id, name=name, label=label)

        if type == -1:
            self.type = random.randint(0,len(self.types)-1)
        else:
            self.type = type
        
        self.population.increment_type_count(self.type)

        self.ds = self.experiment.config.getfloat(self.config_section, 'death_sensitive')
        self.dr = self.experiment.config.getfloat(self.config_section, 'death_resistant')
        self.dp = self.experiment.config.getfloat(self.config_section, 'death_producer')
        self.tp = self.experiment.config.getfloat(self.config_section, 'toxicity')

    def __str__(self):
        """Produce a string to be used when the object is printed"""
        return 'Kerr07 Cell %d Type %d (%s)' % (self.id, self.type, self.types[self.type])

    def type(self):
        """Return the name of the type of this cell"""
        return self.types[self.type]

    def update(self):
        """Update the cell based on its neighbors

        Empty cells will be replaced by a randomly-chosen neighbor cell. In
        this process, that neighbor cell effectively "reproduces" into the
        Empty cell.

        Sensitive cells may die due to their death rate or toxin produced by
        neighbors

        Resistant and Producer cells may die due to their death rate

        """

        typecount = {0: 0, 1: 0, 2: 0, 3: 0}
        neighbors = self.get_neighbors()

        if self.type == self.EMPTY:
            parent = random.choice(neighbors)
            self.type = parent.type
            self.population.update_type_count(self.EMPTY, self.type)            

        elif self.type == self.SENSITIVE:
            for n in neighbors:
                typecount[n.type] += 1
        
            fs = float(typecount[self.SENSITIVE])/len(neighbors)
            fr = float(typecount[self.RESISTANT])/len(neighbors)
            fp = float(typecount[self.PRODUCER])/len(neighbors)
           
            if random.random() < (self.ds + self.tp * fp):
                self.type = self.EMPTY
                self.population.update_type_count(self.SENSITIVE, self.EMPTY)            
                
        elif self.type == self.RESISTANT:
            if random.random() < self.dr:
                self.type = self.EMPTY
                self.population.update_type_count(self.RESISTANT, self.EMPTY)            

        elif self.type == self.PRODUCER:
            if random.random() < self.dp:
                self.type = self.EMPTY
                self.population.update_type_count(self.PRODUCER, self.EMPTY)            

        else:
            print("Error: Invalid cell type %d for cell %d" % (self.type, self.id))




# -*- coding: utf-8 -*-
""" The Experiment encompasses all aspects of the experiment. It maintains the
Actions, Resources, Cells, the configuration, and time.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

import datetime
import os
import random
import shutil
import sys
import time
import uuid

import seeds
from seeds.Cell import *
from seeds.Config import *
from seeds.PluginManager import *
from seeds.Population import *
from seeds.Resource import *
from seeds.SEEDSError import *
from seeds.Topology import *

class Experiment(object):
    """
    The Experiment object captures the state of a experiment

    Properties:

    actions
        A list of Actions to be run sorted by priority.  
    config
        A Config object storing the configuration for the experiment
    data
        A dict that can be used to store additional data.  For example, see the
        'type_count' value, which is used to store the counts of different cell
        types (for cells that have different types) across the population.
        This is faster than scanning the population whenever this information
        is needed.
    epoch
        An integer storing the current epoch (unit of time)
    plugin_manager
        A PluginManager object which manages all Plugins for the experiment
    population
        A Population object that keeps information about the Cells (organisms)
        and their interactions
    proceed
        Boolean value indicating whether or not the experiment should continue.
    resources
        A hash of available resources.  The key is the name of the resource,
        and the value is a Resource object.
    uuid
        A practically unique identifier for the experiment. (RFC 4122 ver 4)
    label
        A unique label identifying the configuration for this Experiment
    config_section
        The section of the config file in which to find settings for this
        Experiment

    """

    def __init__(self, configfile=None, seed=-1, label=None):
        """Initialize a Experiment object

        Parameters:

        *configfile*
            Name of configuration file for the experiment.  If none is
            provided, the experiment will either use defaults for all
            parameters or values provided to the Config object elsewhere (e.g.,
            through the command-line or GUI program).
        *seed*
            Seed for pseudorandom number generator.  If undefined, the current
            time will be used.
        *label*
            A unique string identifying the configuration for this experiment.
            By default, Experiment will look for settings in the [Experiment]
            section of the config file.  If a label is specified, it will look
            in [Experiment:label].

        """

        self.config = Config(experiment=self, filename=configfile)
        self.epoch = 0
        self.is_setup = False
        self.proceed = True
        self.seed = seed
        self.uuid = uuid.uuid4()
        self.data = {}
        self.resources = {}
        self.actions = []
        self.label = label

        if self.label:
            self.config_section = "%s:%s" % ("Experiment", self.label)
        else:
            self.config_section = "%s" % ("Experiment")

        if not self.config.has_section(self.config_section):
            raise ConfigurationError("Configuration section '%s' not defined" % (self.config_section))

    def setup(self):
        """Set up the Experiment including its Population, Resources, and Actions"""
        if self.seed == -1:
            configseed = self.config.getint(self.config_section, "seed", default=-1)
            if configseed != -1:
                self.seed = configseed
            else:
                self.seed = int(time.time()*10)

        random.seed(self.seed)
        self.config.set(self.config_section, 'seed', self.seed)

        self.experiment_epochs = self.config.getint(self.config_section, 'epochs',
                                                    default=-1)

        # Create the data directory.  If the directory already exists, move it
        # to a new directory named after the current name with a timestamp
        # appended
        data_dir = self.config.get(section=self.config_section,
                                   name='data_dir',
                                   default='data')

        if os.path.exists(data_dir):
            newname = data_dir + '-' + datetime.datetime.now().strftime("%Y%m%d%H%M%S")
            shutil.move(data_dir, newname)

        os.mkdir(data_dir)


        # Create a plugin manager.  Append the system-wide plugins
        # to the list of plugin sources.
        self.plugin_manager = PluginManager(self)
        global_plugin_path = os.path.join(os.path.dirname(seeds.__file__), "plugins")
        for d in ["cell", "topology", "action", "resource"]:
            plugin_path = os.path.join(global_plugin_path, d)
            self.plugin_manager.append_dir(plugin_path)


        # Initialize all of the Resources
        self.data['resources'] = {}
        resourcestring = self.config.get(self.config_section, "resources")
        if resourcestring:
            reslist = [res.strip() for res in resourcestring.split(',')]

            for res in reslist:
                sec = "Resource:%s" % (res)
                if not self.config.has_section(sec):
                    raise ConfigurationError("No configuration for resource '%s'" % (res))

                r = Resource(experiment=self, label=res)

                if r.name not in self.resources:
                    self.resources[r.name] = r
                else:
                    print("Warning: Resource '%s' listed twice.  Skipping duplicates." % (res))

        # Create the Population
        self.data['population'] = {}

        population_raw = self.config.get(self.config_section, "population", default="Population")
        parsed = population_raw.split(':')

        if parsed[0] != "Population":
            raise ConfigurationError("Population configuration section name must begin with 'Population'")

        if len(parsed) > 1:
            poplabel = parsed[1]
        else:
            poplabel = None

        try:
            self.population = Population(experiment=self, label=poplabel)
        except TopologyPluginNotFoundError as err:
            raise TopologyPluginNotFoundError(err.topology)
        except CellPluginNotFoundError as err:
            raise CellPluginNotFoundError(err.cell)


        # Setup the list of Actions to be run
        actionstring = self.config.get(section=self.config_section,
                                       name="actions")

        if actionstring:
            actionlist = [action.strip() for action in actionstring.split(',')]

            for item in actionlist:
                parsed = item.split(':')
                action = parsed[0]

                if len(parsed) > 1:
                    label = parsed[1]
                else:
                    label = None

                try:
                    oref = self.plugin_manager.get_plugin(action,
                                                          type=Action)
                    a = oref(self, label=label)
                    self.add_action(a)
                except PluginNotFoundError as err:
                    raise ActionPluginNotFoundError(action)
                except SEEDSError as err:
                    raise SEEDSError(err)


        self.is_setup = True

    def update(self):
        """Update the Experiment and all of its objects"""
        if not self.is_setup:
            self.setup()

        [a.update() for a in self.actions]
        [self.resources[res].update() for res in self.resources]
        self.population.update()
        self.epoch += 1

        # If we've surpassed the configured number of epochs to run for, set
        # proceed to false
        if self.experiment_epochs != -1 and self.epoch >= self.experiment_epochs:
            self.proceed = False

    def __iter__(self):
        """Experiment is an iterator, so it can be used with commands such as

        e = Experiment(....)
        for epoch in e:
            print "Updated Experiment.  Now at epoch %d" % (epoch)
        """

        return self

    def next(self):
        """Proceed with the experiment.  Update the Experiment and return the
        current epoch.  Note: this is for Python 2 support and merely uses the
        __next__ method from Python 3"""

        return self.__next__()

    def __next__(self):
        """Proceed with the experiment.  Update the Experiment and return the current epoch"""
        if not self.proceed:
            raise StopIteration
        else:
            self.update()
            return self.epoch

    def end(self):
        """Set the experiment to end after this epoch"""
        self.proceed = False

    def teardown(self):
        """Perform any necessary cleanup at the end of a run"""
        [a.teardown() for a in self.actions]
        [self.resources[res].teardown() for res in self.resources]
        self.population.teardown()

    def is_resource_defined(self, name):
        """Helper function to determine whether a given resource has been
        defined or not
        """

        return self.resources.has_key(name)

    def get_resource(self, name):
        """Helper function to get the Resource object with the given name.  If
        the resource is not defined, ResourceNotDefinedError is thrown.
        """

        try:
            r = self.resources[name]
            return r
        except KeyError:
            raise ResourceNotDefinedError(name)

    def add_action(self, action):
        """Add an Action to the list of actions to be scheduled.

        Parameters:

        *action*
            An instantiated Action object

        """

        loaded_actions = [a.config_section for a in self.actions]

        if action.config_section in loaded_actions:
            print("Warning: Action '%s' listed twice.  Skipping duplicates." % (action.config_section))
        else:
            self.actions.append(action)
            self.actions = sorted(self.actions, reverse=True, key=lambda a: a.priority)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Script to perform a SEEDS experiment.  For more information, run with the
--help argument.  Plugin directories can be specified in the configuration as
well as in the $SEEDSPLUGINPATH environment variable, which allows for a user-
or system-wide repository of plugins.
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__version__ = "1.0.10"
__credits__ = "Brian Connelly"

import seeds as S
from seeds.SEEDSError import *
from optparse import OptionParser

import os
import re
import sys


class ProgressBar:
    def __init__(self, min_value=0, max_value=0, increment=1, width=80):
        self.min_value = min_value
        self.max_value = max_value
        self.increment = 1
        self.width = width
        self.amount = 0

        self.span = self.max_value - self.min_value
        self.epoch_maxstrlen  = len("epoch: %d" % (self.max_value))

    def update(self, value=None):
        if value:
            self.amount = value
        else:
            self.amount += self.increment

    def __str__(self):
        if self.span > 0:
            usable_width = self.width - self.epoch_maxstrlen - 2 - 1
            tick_width = usable_width / 100.0

            diff = float(self.amount - self.min_value)
            percent_done = int(round((diff / float(self.span)) * 100.0))
            num_ticks = int(round((percent_done * usable_width) / 100))

            return "[%s%s] epoch: %d" % ('#' * num_ticks, ' ' * (usable_width - num_ticks), self.amount)
        else:
            return "epoch: %d" % (self.amount)

def main():
    parser = OptionParser('usage: %prog [options] arg')
    parser.add_option("-c", "--config", dest="configfile", type="string", default="seeds.cfg",
                      help="read config file (default: seeds.cfg)")
    parser.add_option("-C", "--genconfig", action="store_true", dest="genconfig", help="write config file used (experiment.cfg)")
    parser.add_option("-d", "--data_dir", dest="datadir", type="string",
                      help="write data to this directory (default: data)")
    parser.add_option("-e", "--experiment", dest="experiment", type="string", help="label of the experiment to run")
    parser.add_option("-p", "--param", dest="params", type="string", help="Set config values.  Semicolon-separated list of section.param=val")
    parser.add_option("-q", "--quiet", action="store_true", dest="quiet", help="suppress all output messages")
    parser.add_option("-s", "--seed", dest="seed", type=int, default=0,
                      help="set random seed (default: use clock)")
    parser.add_option("-v", "--version", action="store_true", dest="version", help="display version information and quit")

    (cmd_options, cmd_args) = parser.parse_args()

    if cmd_options.seed > 0:
        random_seed = cmd_options.seed
    else:
        random_seed=-1

    if cmd_options.version:
        print("%s (SEEDS Version %s)" % (__version__, S.__version__))
        sys.exit(0)

    if cmd_options.experiment:
        experiment_label = cmd_options.experiment
    else:
        experiment_label = None

    # Create the Experiment...
    try:
        experiment = S.Experiment(configfile=cmd_options.configfile, seed=random_seed,
                                  label=experiment_label)
    except SEEDSError as err:
        print("Error: %s" % err)
        sys.exit(1)

    if cmd_options.datadir:
        experiment.config.set(experiment.config_section, 'data_dir', cmd_options.datadir)


    # Add command-line config options
    if cmd_options.params != None:
        options = re.split("\s*;\s*", cmd_options.params)
        for opt in options:
            # This is perhaps not the best regexp for comma-separated lists as values... need spaces.
            m = re.match(r"(?P<section>[A-Za-z0-9:_]+)\.(?P<parameter>[A-Za-z0-9:_]+)\s*=\s*(?P<value>-?[A-Za-z0-9_\.\,]+)", opt)
            if m != None:
                experiment.config.set(m.group("section"), m.group("parameter"), m.group("value"))
            else:
                print("Error: Could not parse parameter setting", opt)

    # Get the current configured list of plugin directories
    cfg_plugindirs = experiment.config.get(section="Experiment", name="plugin_dirs")
    if cfg_plugindirs:
        plugindirs = experiment.config.get(section="Experiment", name="plugin_dirs").split(",")
    else:
        plugindirs = []

    # Add any plugin paths specified in the environment via $SEEDSPLUGINPATH
    seedspluginpath = os.environ.get("SEEDSPLUGINPATH")
    if seedspluginpath != None and len(seedspluginpath) > 0:
        for p in seedspluginpath.rsplit(":"):
            pdir = os.path.expanduser(p)
            plugindirs.append(pdir)

    if len(plugindirs) > 0:
        pdirs = ",".join(plugindirs)
        experiment.config.set(section="Experiment", name="plugin_dirs", value=pdirs)

    if not cmd_options.quiet:
        print("Experiment ID: %s" % experiment.uuid)

    # Set up a progress bar
    prog = ProgressBar(min_value = 0, max_value=experiment.config.getint(experiment.config_section, 'epochs', default=0))

    # Do the experiment...
    try:
        for epoch in experiment:
            prog.update()
            if not cmd_options.quiet:
                sys.stdout.write("%s\r" % prog)
                sys.stdout.flush()
    except SEEDSError as err:
        print("Error: %s" % err)
        sys.exit(2)
    else:
        experiment.teardown()

    # Write a config file
    if cmd_options.genconfig:
        experiment.config.write(filename='experiment.cfg')

    if not cmd_options.quiet:
        print("")

if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
""" Stop the experiment when the number of existing Cell types in the
population drops below a threshold.  This allows experiments to end earlier
than the configured number if epochs have passed or experiments to not specify
the total number of epochs to observe.  This is accomplished by calling the
Experiment's end method.

"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"

from seeds.Action import *
from seeds.SEEDSError import *


class StopOnConvergence(Action):
    """ Stop the experiment when the number of exisiting cell types in the
    population drops below the configured threshold.

    Configuration is done in the [StopOnConvergence] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    threshold
        The number of cell types at or below which to stop the experiment
        (default: 0)


    Configuration Example:

    [StopOnConvergence]
    threshold = 2
    epoch_start = 0
    epoch_end = 100
    frequency = 1
    priority = 0

    """

    def __init__(self, experiment, label=None):
        """Initialize the StopOnConvergence Action"""

        super(StopOnConvergence, self).__init__(experiment,
                                                name="StopOnConvergence",
                                                label=label)

        self.threshold = self.experiment.config.getint(self.config_section, 'threshold', 0)
        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end',
                                                  default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)

        if self.threshold < 0:
            raise ConfigurationError("StopOnConvergence: threshold value must be at least 0")

    def update(self):
        """Execute the Action"""
        if self.skip_update():
	        return

        converged = True

        num_active_types = 0
        for tcount in self.experiment.data['population']['type_count']:
            if tcount > 0:
                num_active_types += 1

        if num_active_types >= self.threshold:
            converged = False

        if converged:
            self.experiment.end()



# -*- coding: utf-8 -*-
"""
Print the number of Cells for each cell type
"""

__author__ = "Brian Connelly <bdc@msu.edu>"
__credits__ = "Brian Connelly"


import csv

from seeds.Action import *

class PrintCellTypeCount(Action):
    """ Write the number of cells of each type

    Configuration is done in the [PrintCellTypeCount] section

    Configuration Options:

    epoch_start
        The epoch at which to start executing (default: 0)
    epoch_end
        The epoch at which to stop executing (default: end of experiment)
    frequency
        The frequency (epochs) at which to execute (default: 1)
    priority
        The priority of this action.  Actions with higher priority get run
        first.  (default: 0)
    filename
        The name of the file to write to (default: cell_type_count.csv)
    header
        Whether or not to write a header to the output file.  The header will
        be an uncommented, comma-separated list of property names corresponding
        to the data in each row. (default: True)


    Configuration Example:

    [PrintCellTypeCount]
    epoch_start = 3
    epoch_end = 100
    frequency = 2
    priority = 0
    filename = cell_type_count.csv
    header = True

    """

    def __init__(self, experiment, label=None):
        """Initialize the PrintCellTypeCount Action"""

        super(PrintCellTypeCount, self).__init__(experiment,
                                                 name="PrintCellTypeCount",
                                                 label=label)

        self.epoch_start = self.experiment.config.getint(self.config_section, 'epoch_start', 0)
        self.epoch_end = self.experiment.config.getint(self.config_section, 'epoch_end', default=self.experiment.config.getint('Experiment', 'epochs', default=-1))
        self.frequency = self.experiment.config.getint(self.config_section, 'frequency', 1)
        self.priority = self.experiment.config.getint(self.config_section, 'priority', 0)
        self.filename = self.experiment.config.get(self.config_section, 'filename', 'cell_type_count.csv')
        self.header = self.experiment.config.getboolean(self.config_section, 'header', default=True)

        self.types = self.experiment.population._cell_class.types

        data_file = self.datafile_path(self.filename)
        self.writer = csv.writer(open(data_file, 'w'))

        if self.header:
            header = ['epoch']
            header += self.types
            self.writer.writerow(header)

    def update(self):
        """Execute the action"""
        if self.skip_update():
	        return

        row = [self.experiment.epoch] + self.experiment.data['population']['type_count']
        self.writer.writerow(row)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
from distutils.core import setup

import seeds as S

if os.path.exists('MANIFEST'): os.remove('MANIFEST')

if sys.argv[-1] == 'setup.py':
    print("To install, run 'python setup.py install'")
    print()

if sys.version_info[:2] < (2, 6):
    print("SEEDS requires Python version 2.6 or later (%d.%d detected)." %
    sys.version_info[:2])
    sys.exit(-1)

if __name__ == "__main__":

    with open('README.rst') as file:
        ldesc = file.read()

    setup(
        name = "seeds",
        version = S.__version__,
        packages = ['seeds','seeds.plugins','seeds.plugins.cell','seeds.plugins.topology','seeds.plugins.action','seeds.plugins.resource','seeds.utils'],
        scripts = ['scripts/runseeds.py'],
        license = S.__license__,
        author = "Brian Connelly",
        author_email = "bdc@msu.edu",
        maintainer = "Brian Connelly",
        maintainer_email = "bdc@msu.edu",
        url = "https://github.com/briandconnelly/seeds",
        download_url = S.__download_url__,
        keywords = ["simulation","evolution","ecology"],
        classifiers = [
            "Development Status :: 4 - Beta",
            #"Development Status :: 5 - Production/Stable",
            "Intended Audience :: Science/Research",
            "Intended Audience :: Education",
            "License :: OSI Approved :: Apache Software License",
            "Topic :: Scientific/Engineering",
            "Topic :: Scientific/Engineering :: Artificial Life"
        ],
        description = "Stochastic Ecological and Evolutionary Dynamics System",
        long_description = ldesc
)




# -*- coding: utf-8 -*-
"""
Topology in which the location of each node is represented by points randomly
placed on a 2D plane.  Each node is connected to a configured expected number
of neighbors.

This topology was originally presented and used used in the publication:

    B.D. Connelly, L. Zaman, C. Ofria, and P.K. McKinley, "Social structure and
    the maintenance of biodiversity," in Proceedings of the 12th International
    Conference on the Synthesis and Simulation of Living Systems (ALIFE), pp.
    461-468, 2010.

"""

__author__ = "Luis Zaman <zamanlui@msu.edu>"
__credits__ = "Luis Zaman, Brian Connelly, Philip McKinley, Charles Ofria"

import networkx as nx
import random
from math import sqrt, floor, ceil, pi

from seeds.SEEDSError import *
from seeds.Topology import *
from seeds.utils.geometry import euclidean_distance


class CartesianTopology(Topology):
    """
    Topology based on points in Cartesian space on a 2D plane

    Points are placed randomly on a unit two-dimensional Cartesian plane.
    Each node is connected to those nodes that fall within a given distance.
    This distance is calculated to yield an expected number of neighbors
    equal to the expected_neighbors configuration parameter.

    Configuration: All configuration parameters are specified in a
      [CartesianTopology<:label>] block.

    size
        Total number of nodes in the topology.  If size=1, the
        expected_neighbors parameter is ignored, since the one node can not
        have any neighbors. (Integer)
    periodic
        Whether or not to use periodic boundary conditions, which connects the
        edges of the plane, forming a torus. (Boolean.  Default: False)
    expected_neighbors
        The number of neighbors (expected) each node will have. (Default: 0)
    remove_disconnected
        Whether or not to remove nodes that do not have neighbors within the
        calculated radius.  If False, node is connected to a randomly-chosen
        neighbor (Boolean.  Default: True)
    config_section
        The name of the section in which this topology is configured

    Example:

    [CartesianTopology]
    size = 100000
    periodic = True
    expected_neighbors = 20

    """

    def __init__(self, experiment, label=None):
        """Initialize a CartesianTopology object

        Parameters:

        *experiment*
            A reference to the Experiment
        *label*
            A unique string identifying the configuration for this topology

        """

        super(CartesianTopology, self).__init__(experiment, label=label)

        if self.label:
            self.config_section="%s:%s" % ("CartesianTopology", label)
        else:
            self.config_section="%s" % ("CartesianTopology")

        self.size = self.experiment.config.getint(self.config_section, 'size')
        self.periodic = self.experiment.config.getboolean(self.config_section, 'periodic', default=False)
        self.expected_neighbors = self.experiment.config.getint(self.config_section, 'expected_neighbors', default=0)
        self.remove_disconnected = self.experiment.config.getboolean(self.config_section, 'remove_disconnected', default=True)

        if not self.size:
            raise ConfigurationError("CartesianTopology: must specify a size")
        elif self.size < 1:
            raise ConfigurationError("CartesianTopology: size must be >0")
        elif self.expected_neighbors < 0:
            raise ConfigurationError("CartesianTopology: expected_neighbors can not be negative")
        elif self.expected_neighbors > self.size:
            raise ConfigurationError("CartesianTopology: expected_neighbors can not exceed size")

        self.graph = self.build_graph(size=self.size,
                                      expected_neighbors=self.expected_neighbors,
                                      periodic=self.periodic)

        if self.remove_disconnected and len(self.graph.nodes()) < self.size:
            self.relabel_nodes()

    def build_graph(self, size=0, expected_neighbors=0,
                    periodic=False):
        """Build the graph

        Parameters:

        *size*
            The number of nodes to be in the graph
        *expected_neighbors*
            The expected degree of each node in the graph
        *periodic*
            Whether or not to use periodic boundary conditions

        """

        # Calculate the distance required to yield the expected # neighbors
        if size == 1:
            radius = 1
        else:
            radius = sqrt( (expected_neighbors / (size - 1.0)) / pi)

        # Create bins in which to put node so we only check a fraction of
        # candidate neighbors
        num_bins = int(ceil(1/radius))
        neighbor_bins = []
        for i in range(num_bins):
            neighbor_bins.append([])
            for j in range(num_bins):
                neighbor_bins[i].append([])

        G = nx.empty_graph()
        G.name = "Cartesian Topology Graph"
        G.add_nodes_from(list(range(size)))

        # Create the collection of nodes and put them into bins with
        # candidate neighbors
        for n in G.nodes():
            xcoord = random.random()
            ycoord = random.random()
            G.node[n]['coords'] = (xcoord, ycoord)

            # Put node into bin with candidate neighbors
            bin_x = int(floor(xcoord/radius))
            bin_y = int(floor(ycoord/radius))
            neighbor_bins[bin_x][bin_y].append(n)

        # Find actual neighbors and create edges between nodes
        for x in range(num_bins):
            for y in range(num_bins):

                # Get all potential neighbors (those in adjacent bins)
                potentials = []
                for px in range(x-1, x+1+1):
                    if (periodic == False and
                        (px < 0 or px >= num_bins)):
                        continue

                    for py in range(y-y, y+1+1):
                        if (periodic == False and
                            (py < 0 or py >= num_bins)):
                            continue

                        potentials += neighbor_bins[px % num_bins][py % num_bins]

                for node in neighbor_bins[x][y]:
                    node_coords = G.node[node]['coords']
                    for potential in potentials:
                        p_coords = G.node[potential]['coords']
                        if (self.within_range(node_coords, p_coords, radius,
                                              periodic) and
                            node != potential):
                            G.add_edge(node, potential)

                    if G.degree(node) == 0:
                        if self.remove_disconnected:
                            G.remove_node(node)
                            neighbor_bins[x][y].remove(node)
                        else:
                            G.add_edge(node, random.choice(potentials))

                neighbor_bins[x][y] = []

        return G

    def within_range(self, node1, node2, distance, periodic):
        """Determine whether or not two nodes are within a given distance from
        each other

        Parameters:

        *node1*
            The first node (tuple)
        *node2*
            The second node (tuple)
        *distance*
            The threshold distance 
        *periodic*
            Whether or not periodic boundary conditions are used.

        """

        return euclidean_distance(node1, node2, periodic) < distance

    def add_node(self, id=-1, neighbors=[]):
        """Add a node to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_node is not supported by CartesianTopology")
        return

    def remove_node(self, id):
        """Remove a node from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_node is not supported by CartesianTopology")
        return

    def add_edge(self, src, dest):
        """Add an edge to the graph.  Not supported by this topology type"""
        raise ConfigurationError("add_edge is not supported by CartesianTopology")
        return

    def remove_edge(self, src, dest):
        """Remove an edge from the graph.  Not supported by this topology
        type"""
        raise ConfigurationError("remove_edge is not supported by CartesianTopology")
        return


    def relabel_nodes(self):
        """Relabel the nodes in the graph so that labels are numbers from
        0..len(graph) with no gaps.  This is done, for instance, after
        disconnected nodes have been removed from the graph.
        """
        M = {}
        for i in range(len(self.graph.nodes())):
            M[self.graph.nodes()[i]] = i
        
        self.graph = nx.relabel_nodes(self.graph, M)
